@@ -212,7 +212,8 @@
 #define glDrawElementsInstancedANGLE wrap_glDrawElementsInstancedANGLE
 #define glDrawElementsInstancedBaseInstanceEXT wrap_glDrawElementsInstancedBaseInstanceEXT
 #define glDrawElementsInstancedBaseVertex wrap_glDrawElementsInstancedBaseVertex
-#define glDrawElementsInstancedBaseVertexBaseInstanceEXT wrap_glDrawElementsInstancedBaseVertexBaseInstanceEXT
+#define glDrawElementsInstancedBaseVertexBaseInstanceEXT \
+    wrap_glDrawElementsInstancedBaseVertexBaseInstanceEXT
 #define glDrawElementsInstancedBaseVertexEXT wrap_glDrawElementsInstancedBaseVertexEXT
 #define glDrawElementsInstancedBaseVertexOES wrap_glDrawElementsInstancedBaseVertexOES
 #define glDrawElementsInstancedEXT wrap_glDrawElementsInstancedEXT
