@@ -17,10 +17,17 @@
 package android.graphics;
 
 import android.annotation.CheckResult;
+import android.annotation.NonNull;
+import android.annotation.Nullable;
+import android.annotation.UnsupportedAppUsage;
 import android.os.Parcel;
 import android.os.Parcelable;
-
 import android.text.TextUtils;
+import android.util.proto.ProtoInputStream;
+import android.util.proto.ProtoOutputStream;
+import android.util.proto.WireTypeMismatchException;
+
+import java.io.IOException;
 import java.io.PrintWriter;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
@@ -87,7 +94,7 @@ public final class Rect implements Parcelable {
      * @param r The rectangle whose coordinates are copied into the new
      *          rectangle.
      */
-    public Rect(Rect r) {
+    public Rect(@Nullable Rect r) {
         if (r == null) {
             left = top = right = bottom = 0;
         } else {
@@ -98,6 +105,30 @@ public final class Rect implements Parcelable {
         }
     }
 
+    /**
+     * @hide
+     */
+    public Rect(@Nullable Insets r) {
+        if (r == null) {
+            left = top = right = bottom = 0;
+        } else {
+            left = r.left;
+            top = r.top;
+            right = r.right;
+            bottom = r.bottom;
+        }
+    }
+
+    /**
+     * Returns a copy of {@code r} if {@code r} is not {@code null}, or {@code null} otherwise.
+     *
+     * @hide
+     */
+    @Nullable
+    public static Rect copyOrNull(@Nullable Rect r) {
+        return r == null ? null : new Rect(r);
+    }
+
     @Override
     public boolean equals(Object o) {
         if (this == o) return true;
@@ -128,6 +159,7 @@ public final class Rect implements Parcelable {
     /**
      * Return a string representation of the rectangle in a compact form.
      */
+    @NonNull
     public String toShortString() {
         return toShortString(new StringBuilder(32));
     }
@@ -136,7 +168,8 @@ public final class Rect implements Parcelable {
      * Return a string representation of the rectangle in a compact form.
      * @hide
      */
-    public String toShortString(StringBuilder sb) {
+    @NonNull
+    public String toShortString(@NonNull StringBuilder sb) {
         sb.setLength(0);
         sb.append('['); sb.append(left); sb.append(',');
         sb.append(top); sb.append("]["); sb.append(right);
@@ -152,6 +185,7 @@ public final class Rect implements Parcelable {
      * 
      * @return Returns a new String of the form "left top right bottom"
      */
+    @NonNull
     public String flattenToString() {
         StringBuilder sb = new StringBuilder(32);
         // WARNING: Do not change the format of this string, it must be
@@ -170,7 +204,8 @@ public final class Rect implements Parcelable {
      * Returns a Rect from a string of the form returned by {@link #flattenToString},
      * or null if the string is not of that form.
      */
-    public static Rect unflattenFromString(String str) {
+    @Nullable
+    public static Rect unflattenFromString(@Nullable String str) {
         if (TextUtils.isEmpty(str)) {
             return null;
         }
@@ -189,12 +224,64 @@ public final class Rect implements Parcelable {
      * Print short representation to given writer.
      * @hide
      */
-    public void printShortString(PrintWriter pw) {
+    @UnsupportedAppUsage
+    public void printShortString(@NonNull PrintWriter pw) {
         pw.print('['); pw.print(left); pw.print(',');
         pw.print(top); pw.print("]["); pw.print(right);
         pw.print(','); pw.print(bottom); pw.print(']');
     }
-    
+
+    /**
+     * Write to a protocol buffer output stream.
+     * Protocol buffer message definition at {@link android.graphics.RectProto}
+     *
+     * @param protoOutputStream Stream to write the Rect object to.
+     * @param fieldId           Field Id of the Rect as defined in the parent message
+     * @hide
+     */
+    public void writeToProto(@NonNull ProtoOutputStream protoOutputStream, long fieldId) {
+        final long token = protoOutputStream.start(fieldId);
+        protoOutputStream.write(RectProto.LEFT, left);
+        protoOutputStream.write(RectProto.TOP, top);
+        protoOutputStream.write(RectProto.RIGHT, right);
+        protoOutputStream.write(RectProto.BOTTOM, bottom);
+        protoOutputStream.end(token);
+    }
+
+    /**
+     * Read from a protocol buffer input stream.
+     * Protocol buffer message definition at {@link android.graphics.RectProto}
+     *
+     * @param proto     Stream to read the Rect object from.
+     * @param fieldId   Field Id of the Rect as defined in the parent message
+     * @hide
+     */
+    public void readFromProto(@NonNull ProtoInputStream proto, long fieldId) throws IOException,
+            WireTypeMismatchException {
+        final long token = proto.start(fieldId);
+        try {
+            while (proto.nextField() != ProtoInputStream.NO_MORE_FIELDS) {
+                switch (proto.getFieldNumber()) {
+                    case (int) RectProto.LEFT:
+                        left = proto.readInt(RectProto.LEFT);
+                        break;
+                    case (int) RectProto.TOP:
+                        top = proto.readInt(RectProto.TOP);
+                        break;
+                    case (int) RectProto.RIGHT:
+                        right = proto.readInt(RectProto.RIGHT);
+                        break;
+                    case (int) RectProto.BOTTOM:
+                        bottom = proto.readInt(RectProto.BOTTOM);
+                        break;
+                }
+            }
+        } finally {
+            // Let caller handle any exceptions
+            proto.end(token);
+        }
+    }
+
     /**
      * Returns true if the rectangle is empty (left >= right or top >= bottom)
      */
@@ -280,7 +367,7 @@ public final class Rect implements Parcelable {
      * @param src The rectangle whose coordinates are copied into this
      *           rectangle.
      */
-    public void set(Rect src) {
+    public void set(@NonNull Rect src) {
         this.left = src.left;
         this.top = src.top;
         this.right = src.right;
@@ -337,7 +424,19 @@ public final class Rect implements Parcelable {
      * @hide
      * @param insets The rectangle specifying the insets on all side.
      */
-    public void inset(Rect insets) {
+    public void inset(@NonNull Rect insets) {
+        left += insets.left;
+        top += insets.top;
+        right -= insets.right;
+        bottom -= insets.bottom;
+    }
+
+    /**
+     * Insets the rectangle on all sides specified by the dimensions of {@code insets}.
+     * @hide
+     * @param insets The insets to inset the rect by.
+     */
+    public void inset(Insets insets) {
         left += insets.left;
         top += insets.top;
         right -= insets.right;
@@ -403,7 +502,7 @@ public final class Rect implements Parcelable {
      * @return true iff the specified rectangle r is inside or equal to this
      *              rectangle
      */
-    public boolean contains(Rect r) {
+    public boolean contains(@NonNull Rect r) {
                // check for empty first
         return this.left < this.right && this.top < this.bottom
                // now check for containment
@@ -452,10 +551,23 @@ public final class Rect implements Parcelable {
      *              return false and do not change this rectangle.
      */
     @CheckResult
-    public boolean intersect(Rect r) {
+    public boolean intersect(@NonNull Rect r) {
         return intersect(r.left, r.top, r.right, r.bottom);
     }
 
+    /**
+     * If the specified rectangle intersects this rectangle, set this rectangle to that
+     * intersection, otherwise set this rectangle to the empty rectangle.
+     * @see #inset(int, int, int, int) but without checking if the rects overlap.
+     * @hide
+     */
+    public void intersectUnchecked(@NonNull Rect other) {
+        left = Math.max(left, other.left);
+        top = Math.max(top, other.top);
+        right = Math.min(right, other.right);
+        bottom = Math.min(bottom, other.bottom);
+    }
+
     /**
      * If rectangles a and b intersect, return true and set this rectangle to
      * that intersection, otherwise return false and do not change this
@@ -469,7 +581,7 @@ public final class Rect implements Parcelable {
      *              false and do not change this rectangle.
      */
     @CheckResult
-    public boolean setIntersect(Rect a, Rect b) {
+    public boolean setIntersect(@NonNull Rect a, @NonNull Rect b) {
         if (a.left < b.right && b.left < a.right && a.top < b.bottom && b.top < a.bottom) {
             left = Math.max(a.left, b.left);
             top = Math.max(a.top, b.top);
@@ -508,7 +620,7 @@ public final class Rect implements Parcelable {
      * @return true iff the two specified rectangles intersect. In no event are
      *              either of the rectangles modified.
      */
-    public static boolean intersects(Rect a, Rect b) {
+    public static boolean intersects(@NonNull Rect a, @NonNull Rect b) {
         return a.left < b.right && b.left < a.right && a.top < b.bottom && b.top < a.bottom;
     }
 
@@ -545,7 +657,7 @@ public final class Rect implements Parcelable {
      *
      * @param r The rectangle being unioned with this rectangle
      */
-    public void union(Rect r) {
+    public void union(@NonNull Rect r) {
         union(r.left, r.top, r.right, r.bottom);
     }
     
@@ -592,6 +704,7 @@ public final class Rect implements Parcelable {
     /**
      * Parcelable interface methods
      */
+    @Override
     public int describeContents() {
         return 0;
     }
@@ -601,6 +714,7 @@ public final class Rect implements Parcelable {
      * a parcel, use readFromParcel()
      * @param out The parcel to write the rectangle's coordinates into
      */
+    @Override
     public void writeToParcel(Parcel out, int flags) {
         out.writeInt(left);
         out.writeInt(top);
@@ -608,10 +722,11 @@ public final class Rect implements Parcelable {
         out.writeInt(bottom);
     }
 
-    public static final Parcelable.Creator<Rect> CREATOR = new Parcelable.Creator<Rect>() {
+    public static final @android.annotation.NonNull Parcelable.Creator<Rect> CREATOR = new Parcelable.Creator<Rect>() {
         /**
          * Return a new rectangle from the data in the specified parcel.
          */
+        @Override
         public Rect createFromParcel(Parcel in) {
             Rect r = new Rect();
             r.readFromParcel(in);
@@ -621,6 +736,7 @@ public final class Rect implements Parcelable {
         /**
          * Return an array of rectangles of the specified size.
          */
+        @Override
         public Rect[] newArray(int size) {
             return new Rect[size];
         }
@@ -632,7 +748,7 @@ public final class Rect implements Parcelable {
      *
      * @param in The parcel to read the rectangle's coordinates from
      */
-    public void readFromParcel(Parcel in) {
+    public void readFromParcel(@NonNull Parcel in) {
         left = in.readInt();
         top = in.readInt();
         right = in.readInt();
@@ -643,6 +759,7 @@ public final class Rect implements Parcelable {
      * Scales up the rect by the given scale.
      * @hide
      */
+    @UnsupportedAppUsage
     public void scale(float scale) {
         if (scale != 1.0f) {
             left = (int) (left * scale + 0.5f);
