@@ -18,21 +18,22 @@ package android.graphics.drawable;
 
 import android.annotation.NonNull;
 import android.annotation.Nullable;
+import android.annotation.UnsupportedAppUsage;
 import android.content.pm.ActivityInfo.Config;
 import android.content.res.ColorStateList;
 import android.content.res.Resources;
 import android.content.res.Resources.Theme;
 import android.content.res.TypedArray;
-import android.graphics.Bitmap;
+import android.graphics.BlendMode;
 import android.graphics.Canvas;
 import android.graphics.ColorFilter;
 import android.graphics.Outline;
 import android.graphics.PixelFormat;
-import android.graphics.PorterDuff.Mode;
 import android.graphics.Rect;
 import android.util.AttributeSet;
 import android.util.DisplayMetrics;
 import android.util.LayoutDirection;
+import android.util.Log;
 import android.view.Gravity;
 import android.view.View;
 
@@ -42,7 +43,6 @@ import org.xmlpull.v1.XmlPullParser;
 import org.xmlpull.v1.XmlPullParserException;
 
 import java.io.IOException;
-import java.util.Collection;
 
 /**
  * A Drawable that manages an array of other Drawables. These are drawn in array
@@ -68,6 +68,8 @@ import java.util.Collection;
  * @attr ref android.R.styleable#LayerDrawableItem_id
 */
 public class LayerDrawable extends Drawable implements Drawable.Callback {
+    private static final String LOG_TAG = "LayerDrawable";
+
     /**
      * Padding mode used to nest each layer inside the padding of the previous
      * layer.
@@ -91,6 +93,8 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
      */
     public static final int INSET_UNDEFINED = Integer.MIN_VALUE;
 
+    @NonNull
+    @UnsupportedAppUsage
     LayerState mLayerState;
 
     private int[] mPaddingL;
@@ -135,11 +139,14 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
         final ChildDrawable[] r = new ChildDrawable[length];
         for (int i = 0; i < length; i++) {
             r[i] = new ChildDrawable(mLayerState.mDensity);
-            r[i].mDrawable = layers[i];
-            layers[i].setCallback(this);
-            mLayerState.mChildrenChangingConfigurations |= layers[i].getChangingConfigurations();
+            Drawable child = layers[i];
+            r[i].mDrawable = child;
+            if (child != null) {
+                child.setCallback(this);
+                mLayerState.mChildrenChangingConfigurations |= child.getChangingConfigurations();
+            }
         }
-        mLayerState.mNum = length;
+        mLayerState.mNumChildren = length;
         mLayerState.mChildren = r;
 
         ensurePadding();
@@ -156,7 +163,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
      */
     LayerDrawable(@Nullable LayerState state, @Nullable Resources res) {
         mLayerState = createConstantState(state, res);
-        if (mLayerState.mNum > 0) {
+        if (mLayerState.mNumChildren > 0) {
             ensurePadding();
             refreshPadding();
         }
@@ -172,13 +179,9 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
             throws XmlPullParserException, IOException {
         super.inflate(r, parser, attrs, theme);
 
-        final LayerState state = mLayerState;
-        if (state == null) {
-            return;
-        }
-
         // The density may have changed since the last update. This will
         // apply scaling to any existing constant state properties.
+        final LayerState state = mLayerState;
         final int density = Drawable.resolveDensity(r, 0);
         state.setDensity(density);
 
@@ -187,7 +190,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
         a.recycle();
 
         final ChildDrawable[] array = state.mChildren;
-        final int N = state.mNum;
+        final int N = state.mNumChildren;
         for (int i = 0; i < N; i++) {
             final ChildDrawable layer = array[i];
             layer.setDensity(density);
@@ -204,10 +207,6 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
         super.applyTheme(t);
 
         final LayerState state = mLayerState;
-        if (state == null) {
-            return;
-        }
-
         final int density = Drawable.resolveDensity(t.getResources(), 0);
         state.setDensity(density);
 
@@ -219,7 +218,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
         }
 
         final ChildDrawable[] array = state.mChildren;
-        final int N = state.mNum;
+        final int N = state.mNumChildren;
         for (int i = 0; i < N; i++) {
             final ChildDrawable layer = array[i];
             layer.setDensity(density);
@@ -269,7 +268,8 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
 
             // If the layer doesn't have a drawable or unresolved theme
             // attribute for a drawable, attempt to parse one from the child
-            // element.
+            // element. If multiple child elements exist, we'll only use the
+            // first one.
             if (layer.mDrawable == null && (layer.mThemeAttrs == null ||
                     layer.mThemeAttrs[R.styleable.LayerDrawableItem_drawable] == 0)) {
                 while ((type = parser.next()) == XmlPullParser.TEXT) {
@@ -279,13 +279,12 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
                             + ": <item> tag requires a 'drawable' attribute or "
                             + "child tag defining a drawable");
                 }
-                layer.mDrawable = Drawable.createFromXmlInner(r, parser, attrs, theme);
-            }
 
-            if (layer.mDrawable != null) {
+                // We found a child drawable. Take ownership.
+                layer.mDrawable = Drawable.createFromXmlInner(r, parser, attrs, theme);
+                layer.mDrawable.setCallback(this);
                 state.mChildrenChangingConfigurations |=
                         layer.mDrawable.getChangingConfigurations();
-                layer.mDrawable.setCallback(this);
             }
 
             addLayer(layer);
@@ -387,13 +386,25 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
 
         final Drawable dr = a.getDrawable(R.styleable.LayerDrawableItem_drawable);
         if (dr != null) {
+            if (layer.mDrawable != null) {
+                // It's possible that a drawable was already set, in which case
+                // we should clear the callback. We may have also integrated the
+                // drawable's changing configurations, but we don't have enough
+                // information to revert that change.
+                layer.mDrawable.setCallback(null);
+            }
+
+            // Take ownership of the new drawable.
             layer.mDrawable = dr;
+            layer.mDrawable.setCallback(this);
+            state.mChildrenChangingConfigurations |=
+                    layer.mDrawable.getChangingConfigurations();
         }
     }
 
     @Override
     public boolean canApplyTheme() {
-        return (mLayerState != null && mLayerState.canApplyTheme()) || super.canApplyTheme();
+        return mLayerState.canApplyTheme() || super.canApplyTheme();
     }
 
     /**
@@ -406,9 +417,10 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
         }
 
         final ChildDrawable[] layers = mLayerState.mChildren;
-        final int N = mLayerState.mNum;
+        final int N = mLayerState.mNumChildren;
         for (int i = 0; i < N; i++) {
-            if (layers[i].mDrawable.isProjected()) {
+            Drawable childDrawable = layers[i].mDrawable;
+            if (childDrawable != null && childDrawable.isProjected()) {
                 return true;
             }
         }
@@ -422,10 +434,11 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
      * @param layer The layer to add.
      * @return The index of the layer.
      */
+    @UnsupportedAppUsage
     int addLayer(@NonNull ChildDrawable layer) {
         final LayerState st = mLayerState;
         final int N = st.mChildren != null ? st.mChildren.length : 0;
-        final int i = st.mNum;
+        final int i = st.mNumChildren;
         if (i >= N) {
             final ChildDrawable[] nu = new ChildDrawable[N + 10];
             if (i > 0) {
@@ -436,7 +449,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
         }
 
         st.mChildren[i] = layer;
-        st.mNum++;
+        st.mNumChildren++;
         st.invalidateCache();
         return i;
     }
@@ -504,7 +517,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
      */
     public Drawable findDrawableByLayerId(int id) {
         final ChildDrawable[] layers = mLayerState.mChildren;
-        for (int i = mLayerState.mNum - 1; i >= 0; i--) {
+        for (int i = mLayerState.mNumChildren - 1; i >= 0; i--) {
             if (layers[i].mId == id) {
                 return layers[i].mDrawable;
             }
@@ -539,7 +552,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
      * @attr ref android.R.styleable#LayerDrawableItem_id
      */
     public int getId(int index) {
-        if (index >= mLayerState.mNum) {
+        if (index >= mLayerState.mNumChildren) {
             throw new IndexOutOfBoundsException();
         }
         return mLayerState.mChildren[index].mId;
@@ -551,7 +564,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
      * @return The number of layers.
      */
     public int getNumberOfLayers() {
-        return mLayerState.mNum;
+        return mLayerState.mNumChildren;
     }
 
     /**
@@ -583,7 +596,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
      */
     public int findIndexByLayerId(int id) {
         final ChildDrawable[] layers = mLayerState.mChildren;
-        final int N = mLayerState.mNum;
+        final int N = mLayerState.mNumChildren;
         for (int i = 0; i < N; i++) {
             final ChildDrawable childDrawable = layers[i];
             if (childDrawable.mId == id) {
@@ -605,7 +618,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
      * @attr ref android.R.styleable#LayerDrawableItem_drawable
      */
     public void setDrawable(int index, Drawable drawable) {
-        if (index >= mLayerState.mNum) {
+        if (index >= mLayerState.mNumChildren) {
             throw new IndexOutOfBoundsException();
         }
 
@@ -641,7 +654,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
      * @attr ref android.R.styleable#LayerDrawableItem_drawable
      */
     public Drawable getDrawable(int index) {
-        if (index >= mLayerState.mNum) {
+        if (index >= mLayerState.mNumChildren) {
             throw new IndexOutOfBoundsException();
         }
         return mLayerState.mChildren[index].mDrawable;
@@ -976,6 +989,11 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
         if (mSuspendChildInvalidation) {
             mChildRequestedInvalidation = true;
         } else {
+            // This may have been called as the result of a tint changing, in
+            // which case we may need to refresh the cached statefulness or
+            // opacity.
+            mLayerState.invalidateCache();
+
             invalidateSelf();
         }
     }
@@ -993,7 +1011,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
     @Override
     public void draw(Canvas canvas) {
         final ChildDrawable[] array = mLayerState.mChildren;
-        final int N = mLayerState.mNum;
+        final int N = mLayerState.mNumChildren;
         for (int i = 0; i < N; i++) {
             final Drawable dr = array[i].mDrawable;
             if (dr != null) {
@@ -1220,7 +1238,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
 
         // Add all the padding.
         final ChildDrawable[] array = mLayerState.mChildren;
-        final int N = mLayerState.mNum;
+        final int N = mLayerState.mNumChildren;
         for (int i = 0; i < N; i++) {
             refreshChildPadding(i, array[i]);
 
@@ -1239,7 +1257,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
 
         // Take the max padding.
         final ChildDrawable[] array = mLayerState.mChildren;
-        final int N = mLayerState.mNum;
+        final int N = mLayerState.mNumChildren;
         for (int i = 0; i < N; i++) {
             refreshChildPadding(i, array[i]);
 
@@ -1258,7 +1276,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
     @Override
     public void getOutline(@NonNull Outline outline) {
         final ChildDrawable[] array = mLayerState.mChildren;
-        final int N = mLayerState.mNum;
+        final int N = mLayerState.mNumChildren;
         for (int i = 0; i < N; i++) {
             final Drawable dr = array[i].mDrawable;
             if (dr != null) {
@@ -1273,7 +1291,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
     @Override
     public void setHotspot(float x, float y) {
         final ChildDrawable[] array = mLayerState.mChildren;
-        final int N = mLayerState.mNum;
+        final int N = mLayerState.mNumChildren;
         for (int i = 0; i < N; i++) {
             final Drawable dr = array[i].mDrawable;
             if (dr != null) {
@@ -1285,7 +1303,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
     @Override
     public void setHotspotBounds(int left, int top, int right, int bottom) {
         final ChildDrawable[] array = mLayerState.mChildren;
-        final int N = mLayerState.mNum;
+        final int N = mLayerState.mNumChildren;
         for (int i = 0; i < N; i++) {
             final Drawable dr = array[i].mDrawable;
             if (dr != null) {
@@ -1313,7 +1331,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
     public boolean setVisible(boolean visible, boolean restart) {
         final boolean changed = super.setVisible(visible, restart);
         final ChildDrawable[] array = mLayerState.mChildren;
-        final int N = mLayerState.mNum;
+        final int N = mLayerState.mNumChildren;
         for (int i = 0; i < N; i++) {
             final Drawable dr = array[i].mDrawable;
             if (dr != null) {
@@ -1327,7 +1345,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
     @Override
     public void setDither(boolean dither) {
         final ChildDrawable[] array = mLayerState.mChildren;
-        final int N = mLayerState.mNum;
+        final int N = mLayerState.mNumChildren;
         for (int i = 0; i < N; i++) {
             final Drawable dr = array[i].mDrawable;
             if (dr != null) {
@@ -1339,7 +1357,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
     @Override
     public void setAlpha(int alpha) {
         final ChildDrawable[] array = mLayerState.mChildren;
-        final int N = mLayerState.mNum;
+        final int N = mLayerState.mNumChildren;
         for (int i = 0; i < N; i++) {
             final Drawable dr = array[i].mDrawable;
             if (dr != null) {
@@ -1361,7 +1379,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
     @Override
     public void setColorFilter(ColorFilter colorFilter) {
         final ChildDrawable[] array = mLayerState.mChildren;
-        final int N = mLayerState.mNum;
+        final int N = mLayerState.mNumChildren;
         for (int i = 0; i < N; i++) {
             final Drawable dr = array[i].mDrawable;
             if (dr != null) {
@@ -1373,7 +1391,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
     @Override
     public void setTintList(ColorStateList tint) {
         final ChildDrawable[] array = mLayerState.mChildren;
-        final int N = mLayerState.mNum;
+        final int N = mLayerState.mNumChildren;
         for (int i = 0; i < N; i++) {
             final Drawable dr = array[i].mDrawable;
             if (dr != null) {
@@ -1383,20 +1401,20 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
     }
 
     @Override
-    public void setTintMode(Mode tintMode) {
+    public void setTintBlendMode(@NonNull BlendMode blendMode) {
         final ChildDrawable[] array = mLayerState.mChildren;
-        final int N = mLayerState.mNum;
+        final int N = mLayerState.mNumChildren;
         for (int i = 0; i < N; i++) {
             final Drawable dr = array[i].mDrawable;
             if (dr != null) {
-                dr.setTintMode(tintMode);
+                dr.setTintBlendMode(blendMode);
             }
         }
     }
 
     private Drawable getFirstNonNullDrawable() {
         final ChildDrawable[] array = mLayerState.mChildren;
-        final int N = mLayerState.mNum;
+        final int N = mLayerState.mNumChildren;
         for (int i = 0; i < N; i++) {
             final Drawable dr = array[i].mDrawable;
             if (dr != null) {
@@ -1434,7 +1452,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
         mLayerState.mAutoMirrored = mirrored;
 
         final ChildDrawable[] array = mLayerState.mChildren;
-        final int N = mLayerState.mNum;
+        final int N = mLayerState.mNumChildren;
         for (int i = 0; i < N; i++) {
             final Drawable dr = array[i].mDrawable;
             if (dr != null) {
@@ -1451,7 +1469,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
     @Override
     public void jumpToCurrentState() {
         final ChildDrawable[] array = mLayerState.mChildren;
-        final int N = mLayerState.mNum;
+        final int N = mLayerState.mNumChildren;
         for (int i = 0; i < N; i++) {
             final Drawable dr = array[i].mDrawable;
             if (dr != null) {
@@ -1465,12 +1483,18 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
         return mLayerState.isStateful();
     }
 
+    /** @hide */
+    @Override
+    public boolean hasFocusStateSpecified() {
+        return mLayerState.hasFocusStateSpecified();
+    }
+
     @Override
     protected boolean onStateChange(int[] state) {
         boolean changed = false;
 
         final ChildDrawable[] array = mLayerState.mChildren;
-        final int N = mLayerState.mNum;
+        final int N = mLayerState.mNumChildren;
         for (int i = 0; i < N; i++) {
             final Drawable dr = array[i].mDrawable;
             if (dr != null && dr.isStateful() && dr.setState(state)) {
@@ -1491,7 +1515,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
         boolean changed = false;
 
         final ChildDrawable[] array = mLayerState.mChildren;
-        final int N = mLayerState.mNum;
+        final int N = mLayerState.mNumChildren;
         for (int i = 0; i < N; i++) {
             final Drawable dr = array[i].mDrawable;
             if (dr != null && dr.setLevel(level)) {
@@ -1533,7 +1557,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
         final boolean isPaddingNested = mLayerState.mPaddingMode == PADDING_MODE_NEST;
         final ChildDrawable[] array = mLayerState.mChildren;
 
-        for (int i = 0, count = mLayerState.mNum; i < count; i++) {
+        for (int i = 0, count = mLayerState.mNumChildren; i < count; i++) {
             final ChildDrawable r = array[i];
             final Drawable d = r.mDrawable;
             if (d == null) {
@@ -1632,7 +1656,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
         final boolean nest = mLayerState.mPaddingMode == PADDING_MODE_NEST;
         final boolean isLayoutRtl = getLayoutDirection() == LayoutDirection.RTL;
         final ChildDrawable[] array = mLayerState.mChildren;
-        final int N = mLayerState.mNum;
+        final int N = mLayerState.mNumChildren;
         for (int i = 0; i < N; i++) {
             final ChildDrawable r = array[i];
             if (r.mDrawable == null) {
@@ -1674,7 +1698,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
 
         final boolean nest = mLayerState.mPaddingMode == PADDING_MODE_NEST;
         final ChildDrawable[] array = mLayerState.mChildren;
-        final int N = mLayerState.mNum;
+        final int N = mLayerState.mNumChildren;
         for (int i = 0; i < N; i++) {
             final ChildDrawable r = array[i];
             if (r.mDrawable == null) {
@@ -1722,8 +1746,9 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
     /**
      * Ensures the child padding caches are large enough.
      */
+    @UnsupportedAppUsage
     void ensurePadding() {
-        final int N = mLayerState.mNum;
+        final int N = mLayerState.mNumChildren;
         if (mPaddingL != null && mPaddingL.length >= N) {
             return;
         }
@@ -1735,7 +1760,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
     }
 
     void refreshPadding() {
-        final int N = mLayerState.mNum;
+        final int N = mLayerState.mNumChildren;
         final ChildDrawable[] array = mLayerState.mChildren;
         for (int i = 0; i < N; i++) {
             refreshChildPadding(i, array[i]);
@@ -1756,7 +1781,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
         if (!mMutated && super.mutate() == this) {
             mLayerState = createConstantState(mLayerState, null);
             final ChildDrawable[] array = mLayerState.mChildren;
-            final int N = mLayerState.mNum;
+            final int N = mLayerState.mNumChildren;
             for (int i = 0; i < N; i++) {
                 final Drawable dr = array[i].mDrawable;
                 if (dr != null) {
@@ -1775,7 +1800,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
         super.clearMutated();
 
         final ChildDrawable[] array = mLayerState.mChildren;
-        final int N = mLayerState.mNum;
+        final int N = mLayerState.mNumChildren;
         for (int i = 0; i < N; i++) {
             final Drawable dr = array[i].mDrawable;
             if (dr != null) {
@@ -1790,7 +1815,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
         boolean changed = false;
 
         final ChildDrawable[] array = mLayerState.mChildren;
-        final int N = mLayerState.mNum;
+        final int N = mLayerState.mNumChildren;
         for (int i = 0; i < N; i++) {
             final Drawable dr = array[i].mDrawable;
             if (dr != null) {
@@ -1803,6 +1828,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
     }
 
     static class ChildDrawable {
+        @UnsupportedAppUsage
         public Drawable mDrawable;
         public int[] mThemeAttrs;
         public int mDensity = DisplayMetrics.DENSITY_DEFAULT;
@@ -1826,15 +1852,24 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
                 final ConstantState cs = dr.getConstantState();
                 if (cs == null) {
                     clone = dr;
+                    if (dr.getCallback() != null) {
+                        // This drawable already has an owner.
+                        Log.w(LOG_TAG, "Invalid drawable added to LayerDrawable! Drawable already "
+                                + "belongs to another owner but does not expose a constant state.",
+                                new RuntimeException());
+                    }
                 } else if (res != null) {
                     clone = cs.newDrawable(res);
                 } else {
                     clone = cs.newDrawable();
                 }
-                clone.setCallback(owner);
                 clone.setLayoutDirection(dr.getLayoutDirection());
                 clone.setBounds(dr.getBounds());
                 clone.setLevel(dr.getLevel());
+
+                // Set the callback last to prevent invalidation from
+                // propagating before the constant state has been set.
+                clone.setCallback(owner);
             } else {
                 clone = null;
             }
@@ -1895,7 +1930,8 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
     static class LayerState extends ConstantState {
         private int[] mThemeAttrs;
 
-        int mNum;
+        int mNumChildren;
+        @UnsupportedAppUsage
         ChildDrawable[] mChildren;
 
         int mDensity;
@@ -1912,10 +1948,10 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
         @Config int mChangingConfigurations;
         @Config int mChildrenChangingConfigurations;
 
-        private boolean mHaveOpacity;
+        private boolean mCheckedOpacity;
         private int mOpacity;
 
-        private boolean mHaveIsStateful;
+        private boolean mCheckedStateful;
         private boolean mIsStateful;
 
         private boolean mAutoMirrored = false;
@@ -1928,9 +1964,9 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
 
             if (orig != null) {
                 final ChildDrawable[] origChildDrawable = orig.mChildren;
-                final int N = orig.mNum;
+                final int N = orig.mNumChildren;
 
-                mNum = N;
+                mNumChildren = N;
                 mChildren = new ChildDrawable[N];
 
                 mChangingConfigurations = orig.mChangingConfigurations;
@@ -1941,9 +1977,9 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
                     mChildren[i] = new ChildDrawable(or, owner, res);
                 }
 
-                mHaveOpacity = orig.mHaveOpacity;
+                mCheckedOpacity = orig.mCheckedOpacity;
                 mOpacity = orig.mOpacity;
-                mHaveIsStateful = orig.mHaveIsStateful;
+                mCheckedStateful = orig.mCheckedStateful;
                 mIsStateful = orig.mIsStateful;
                 mAutoMirrored = orig.mAutoMirrored;
                 mPaddingMode = orig.mPaddingMode;
@@ -1960,7 +1996,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
                     applyDensityScaling(orig.mDensity, mDensity);
                 }
             } else {
-                mNum = 0;
+                mNumChildren = 0;
                 mChildren = null;
             }
         }
@@ -2012,7 +2048,7 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
             }
 
             final ChildDrawable[] array = mChildren;
-            final int N = mNum;
+            final int N = mNumChildren;
             for (int i = 0; i < N; i++) {
                 final ChildDrawable layer = array[i];
                 if (layer.canApplyTheme()) {
@@ -2040,12 +2076,12 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
         }
 
         public final int getOpacity() {
-            if (mHaveOpacity) {
+            if (mCheckedOpacity) {
                 return mOpacity;
             }
 
+            final int N = mNumChildren;
             final ChildDrawable[] array = mChildren;
-            final int N = mNum;
 
             // Seek to the first non-null drawable.
             int firstIndex = -1;
@@ -2072,17 +2108,17 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
             }
 
             mOpacity = op;
-            mHaveOpacity = true;
+            mCheckedOpacity = true;
             return op;
         }
 
         public final boolean isStateful() {
-            if (mHaveIsStateful) {
+            if (mCheckedStateful) {
                 return mIsStateful;
             }
 
+            final int N = mNumChildren;
             final ChildDrawable[] array = mChildren;
-            final int N = mNum;
             boolean isStateful = false;
             for (int i = 0; i < N; i++) {
                 final Drawable dr = array[i].mDrawable;
@@ -2093,13 +2129,25 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
             }
 
             mIsStateful = isStateful;
-            mHaveIsStateful = true;
+            mCheckedStateful = true;
             return isStateful;
         }
 
+        public final boolean hasFocusStateSpecified() {
+            final int N = mNumChildren;
+            final ChildDrawable[] array = mChildren;
+            for (int i = 0; i < N; i++) {
+                final Drawable dr = array[i].mDrawable;
+                if (dr != null && dr.hasFocusStateSpecified()) {
+                    return true;
+                }
+            }
+            return false;
+        }
+
         public final boolean canConstantState() {
             final ChildDrawable[] array = mChildren;
-            final int N = mNum;
+            final int N = mNumChildren;
             for (int i = 0; i < N; i++) {
                 final Drawable dr = array[i].mDrawable;
                 if (dr != null && dr.getConstantState() == null) {
@@ -2111,27 +2159,14 @@ public class LayerDrawable extends Drawable implements Drawable.Callback {
             return true;
         }
 
-        public void invalidateCache() {
-            mHaveOpacity = false;
-            mHaveIsStateful = false;
+        /**
+         * Invalidates the cached opacity and statefulness.
+         */
+        void invalidateCache() {
+            mCheckedOpacity = false;
+            mCheckedStateful = false;
         }
 
-        @Override
-        public int addAtlasableBitmaps(Collection<Bitmap> atlasList) {
-            final ChildDrawable[] array = mChildren;
-            final int N = mNum;
-            int pixelCount = 0;
-            for (int i = 0; i < N; i++) {
-                final Drawable dr = array[i].mDrawable;
-                if (dr != null) {
-                    final ConstantState state = dr.getConstantState();
-                    if (state != null) {
-                        pixelCount += state.addAtlasableBitmaps(atlasList);
-                    }
-                }
-            }
-            return pixelCount;
-        }
     }
 }
 
