@@ -16,9 +16,7 @@
 
 package android.hardware.location;
 
-
 import android.annotation.NonNull;
-import android.annotation.Nullable;
 import android.annotation.SystemApi;
 import android.os.Parcel;
 import android.os.Parcelable;
@@ -26,29 +24,43 @@ import android.os.Parcelable;
 import libcore.util.EmptyArray;
 
 /**
+ * Describes an instance of a nanoapp, used by the internal state managed by ContextHubService.
+ *
+ * TODO(b/69270990) Remove this class once the old API is deprecated.
+ *
+ * @deprecated Use {@link android.hardware.location.NanoAppState} instead.
+ *
  * @hide
  */
 @SystemApi
-public class NanoAppInstanceInfo {
-    private String mPublisher;
-    private String mName;
+@Deprecated
+public class NanoAppInstanceInfo implements Parcelable {
+    private String mPublisher = "Unknown";
+    private String mName = "Unknown";
 
+    private int mHandle;
     private long mAppId;
     private int mAppVersion;
-
-    private int mNeededReadMemBytes;
-    private int mNeededWriteMemBytes;
-    private int mNeededExecMemBytes;
-
-    private int[] mNeededSensors;
-    private int[] mOutputEvents;
-
     private int mContexthubId;
-    private int mHandle;
+
+    private int mNeededReadMemBytes = 0;
+    private int mNeededWriteMemBytes = 0;
+    private int mNeededExecMemBytes = 0;
+
+    private int[] mNeededSensors = EmptyArray.INT;
+    private int[] mOutputEvents = EmptyArray.INT;
 
     public NanoAppInstanceInfo() {
-        mNeededSensors = EmptyArray.INT;
-        mOutputEvents = EmptyArray.INT;
+    }
+
+    /**
+     * @hide
+     */
+    public NanoAppInstanceInfo(int handle, long appId, int appVersion, int contextHubId) {
+        mHandle = handle;
+        mAppId = appId;
+        mAppVersion = appVersion;
+        mContexthubId = contextHubId;
     }
 
     /**
@@ -60,18 +72,6 @@ public class NanoAppInstanceInfo {
         return mPublisher;
     }
 
-
-    /**
-     * set the publisher name for the app
-     *
-     * @param publisher - name of the publisher
-     *
-     * @hide
-     */
-    public void setPublisher(String publisher) {
-        mPublisher = publisher;
-    }
-
     /**
      * get the name of the app
      *
@@ -81,17 +81,6 @@ public class NanoAppInstanceInfo {
         return mName;
     }
 
-    /**
-     * set the name of the app
-     *
-     * @param name - name of the app
-     *
-     * @hide
-     */
-    public void setName(String name) {
-        mName = name;
-    }
-
     /**
      * Get the application identifier
      *
@@ -101,42 +90,15 @@ public class NanoAppInstanceInfo {
         return mAppId;
     }
 
-    /**
-     * Set the application identifier
-     *
-     * @param appId - application identifier
-     *
-     * @hide
-     */
-    public void setAppId(long appId) {
-        mAppId = appId;
-    }
-
     /**
      * Get the application version
      *
-     * NOTE: There is a race condition where shortly after loading, this
-     * may return -1 instead of the correct version.
-     *
-     * TODO(b/30970527): Fix this race condition.
-     *
      * @return int - version of the app
      */
     public int getAppVersion() {
         return mAppVersion;
     }
 
-    /**
-     * Set the application version
-     *
-     * @param appVersion - version of the app
-     *
-     * @hide
-     */
-    public void setAppVersion(int appVersion) {
-        mAppVersion = appVersion;
-    }
-
     /**
      * Get the read memory needed by the app
      *
@@ -146,17 +108,6 @@ public class NanoAppInstanceInfo {
         return mNeededReadMemBytes;
     }
 
-    /**
-     * Set the read memory needed by the app
-     *
-     * @param neededReadMemBytes - readable Memory needed in bytes
-     *
-     * @hide
-     */
-    public void setNeededReadMemBytes(int neededReadMemBytes) {
-        mNeededReadMemBytes = neededReadMemBytes;
-    }
-
     /**
      *  get writable memory needed by the app
      *
@@ -166,18 +117,6 @@ public class NanoAppInstanceInfo {
         return mNeededWriteMemBytes;
     }
 
-    /**
-     * set writable memory needed by the app
-     *
-     * @param neededWriteMemBytes - writable memory needed by the
-     *                            app
-     *
-     * @hide
-     */
-    public void setNeededWriteMemBytes(int neededWriteMemBytes) {
-        mNeededWriteMemBytes = neededWriteMemBytes;
-    }
-
     /**
      * get executable memory needed by the app
      *
@@ -187,18 +126,6 @@ public class NanoAppInstanceInfo {
         return mNeededExecMemBytes;
     }
 
-    /**
-     * set executable memory needed by the app
-     *
-     * @param neededExecMemBytes - executable memory needed by the
-     *                           app
-     *
-     * @hide
-     */
-    public void setNeededExecMemBytes(int neededExecMemBytes) {
-        mNeededExecMemBytes = neededExecMemBytes;
-    }
-
     /**
      * Get the sensors needed by this app
      *
@@ -209,17 +136,6 @@ public class NanoAppInstanceInfo {
         return mNeededSensors;
     }
 
-    /**
-     * set the sensors needed by this app
-     *
-     * @param neededSensors - all the sensors needed by this app
-     *
-     * @hide
-     */
-    public void setNeededSensors(@Nullable int[] neededSensors) {
-        mNeededSensors = neededSensors != null ? neededSensors : EmptyArray.INT;
-    }
-
     /**
      * get the events generated by this app
      *
@@ -230,18 +146,6 @@ public class NanoAppInstanceInfo {
         return mOutputEvents;
     }
 
-    /**
-     * set the output events that can be generated by this app
-     *
-     * @param outputEvents - the events that may be generated by
-     *                     this app
-     *
-     * @hide
-     */
-    public void setOutputEvents(@Nullable int[] outputEvents) {
-        mOutputEvents = outputEvents != null ? outputEvents : EmptyArray.INT;
-    }
-
     /**
      * get the context hub identifier
      *
@@ -251,17 +155,6 @@ public class NanoAppInstanceInfo {
         return mContexthubId;
     }
 
-    /**
-     * set the context hub identifier
-     *
-     * @param contexthubId - system wide unique identifier
-     *
-     * @hide
-     */
-    public void setContexthubId(int contexthubId) {
-        mContexthubId = contexthubId;
-    }
-
     /**
      * get a handle to the nano app instance
      *
@@ -271,18 +164,6 @@ public class NanoAppInstanceInfo {
         return mHandle;
     }
 
-    /**
-     * set the handle for an app instance
-     *
-     * @param handle - handle to this instance
-     *
-     * @hide
-     */
-    public void setHandle(int handle) {
-        mHandle = handle;
-    }
-
-
     private NanoAppInstanceInfo(Parcel in) {
         mPublisher = in.readString();
         mName = in.readString();
@@ -327,7 +208,7 @@ public class NanoAppInstanceInfo {
         out.writeIntArray(mOutputEvents);
     }
 
-    public static final Parcelable.Creator<NanoAppInstanceInfo> CREATOR
+    public static final @android.annotation.NonNull Parcelable.Creator<NanoAppInstanceInfo> CREATOR
             = new Parcelable.Creator<NanoAppInstanceInfo>() {
         public NanoAppInstanceInfo createFromParcel(Parcel in) {
             return new NanoAppInstanceInfo(in);
@@ -342,9 +223,7 @@ public class NanoAppInstanceInfo {
     public String toString() {
         String retVal = "handle : " + mHandle;
         retVal += ", Id : 0x" + Long.toHexString(mAppId);
-        retVal += ", Version : " + mAppVersion;
-        retVal += ", Name : " + mName;
-        retVal += ", Publisher : " + mPublisher;
+        retVal += ", Version : 0x" + Integer.toHexString(mAppVersion);
 
         return retVal;
     }
