@@ -16,6 +16,7 @@
 
 package android.location;
 
+import android.annotation.FloatRange;
 import android.annotation.TestApi;
 import android.os.Parcel;
 import android.os.Parcelable;
@@ -36,6 +37,8 @@ public final class GnssClock implements Parcelable {
     private static final int HAS_BIAS_UNCERTAINTY = (1<<4);
     private static final int HAS_DRIFT = (1<<5);
     private static final int HAS_DRIFT_UNCERTAINTY = (1<<6);
+    private static final int HAS_ELAPSED_REALTIME_NANOS = (1 << 7);
+    private static final int HAS_ELAPSED_REALTIME_UNCERTAINTY_NANOS = (1 << 8);
 
     // End enumerations in sync with gps.h
 
@@ -49,6 +52,8 @@ public final class GnssClock implements Parcelable {
     private double mDriftNanosPerSecond;
     private double mDriftUncertaintyNanosPerSecond;
     private int mHardwareClockDiscontinuityCount;
+    private long mElapsedRealtimeNanos;
+    private double mElapsedRealtimeUncertaintyNanos;
 
     /**
      * @hide
@@ -74,6 +79,8 @@ public final class GnssClock implements Parcelable {
         mDriftNanosPerSecond = clock.mDriftNanosPerSecond;
         mDriftUncertaintyNanosPerSecond = clock.mDriftUncertaintyNanosPerSecond;
         mHardwareClockDiscontinuityCount = clock.mHardwareClockDiscontinuityCount;
+        mElapsedRealtimeNanos = clock.mElapsedRealtimeNanos;
+        mElapsedRealtimeUncertaintyNanos = clock.mElapsedRealtimeUncertaintyNanos;
     }
 
     /**
@@ -167,6 +174,7 @@ public final class GnssClock implements Parcelable {
      * <p>This value is often effectively zero (it is the reference clock by which all other times
      * and time uncertainties are measured), and thus this field may often be 0, or not provided.
      */
+    @FloatRange(from = 0.0f)
     public double getTimeUncertaintyNanos() {
         return mTimeUncertaintyNanos;
     }
@@ -176,7 +184,7 @@ public final class GnssClock implements Parcelable {
      * @hide
      */
     @TestApi
-    public void setTimeUncertaintyNanos(double timeUncertaintyNanos) {
+    public void setTimeUncertaintyNanos(@FloatRange(from = 0.0f) double timeUncertaintyNanos) {
         setFlag(HAS_TIME_UNCERTAINTY);
         mTimeUncertaintyNanos = timeUncertaintyNanos;
     }
@@ -297,6 +305,7 @@ public final class GnssClock implements Parcelable {
      *
      * <p>The value is only available if {@link #hasBiasUncertaintyNanos()} is {@code true}.
      */
+    @FloatRange(from = 0.0f)
     public double getBiasUncertaintyNanos() {
         return mBiasUncertaintyNanos;
     }
@@ -306,7 +315,7 @@ public final class GnssClock implements Parcelable {
      * @hide
      */
     @TestApi
-    public void setBiasUncertaintyNanos(double biasUncertaintyNanos) {
+    public void setBiasUncertaintyNanos(@FloatRange(from = 0.0f) double biasUncertaintyNanos) {
         setFlag(HAS_BIAS_UNCERTAINTY);
         mBiasUncertaintyNanos = biasUncertaintyNanos;
     }
@@ -332,6 +341,9 @@ public final class GnssClock implements Parcelable {
     /**
      * Gets the clock's Drift in nanoseconds per second.
      *
+     * <p>This value is the instantaneous time-derivative of the value provided by
+     * {@link #getBiasNanos()}.
+     *
      * <p>A positive value indicates that the frequency is higher than the nominal (e.g. GPS master
      * clock) frequency. The error estimate for this reported drift is
      * {@link #getDriftUncertaintyNanosPerSecond()}.
@@ -376,6 +388,7 @@ public final class GnssClock implements Parcelable {
      * <p>The value is only available if {@link #hasDriftUncertaintyNanosPerSecond()} is
      * {@code true}.
      */
+    @FloatRange(from = 0.0f)
     public double getDriftUncertaintyNanosPerSecond() {
         return mDriftUncertaintyNanosPerSecond;
     }
@@ -385,7 +398,8 @@ public final class GnssClock implements Parcelable {
      * @hide
      */
     @TestApi
-    public void setDriftUncertaintyNanosPerSecond(double driftUncertaintyNanosPerSecond) {
+    public void setDriftUncertaintyNanosPerSecond(
+            @FloatRange(from = 0.0f) double driftUncertaintyNanosPerSecond) {
         setFlag(HAS_DRIFT_UNCERTAINTY);
         mDriftUncertaintyNanosPerSecond = driftUncertaintyNanosPerSecond;
     }
@@ -400,6 +414,90 @@ public final class GnssClock implements Parcelable {
         mDriftUncertaintyNanosPerSecond = Double.NaN;
     }
 
+    /**
+     * Returns {@code true} if {@link #getElapsedRealtimeNanos()} is available, {@code false}
+     * otherwise.
+     */
+    public boolean hasElapsedRealtimeNanos() {
+        return isFlagSet(HAS_ELAPSED_REALTIME_NANOS);
+    }
+
+    /**
+     * Returns the elapsed real-time of this clock since system boot, in nanoseconds.
+     *
+     * <p>The value is only available if {@link #hasElapsedRealtimeNanos()} is
+     * {@code true}.
+     */
+    public long getElapsedRealtimeNanos() {
+        return mElapsedRealtimeNanos;
+    }
+
+    /**
+     * Sets the elapsed real-time of this clock since system boot, in nanoseconds.
+     * @hide
+     */
+    @TestApi
+    public void setElapsedRealtimeNanos(long elapsedRealtimeNanos) {
+        setFlag(HAS_ELAPSED_REALTIME_NANOS);
+        mElapsedRealtimeNanos = elapsedRealtimeNanos;
+    }
+
+    /**
+     * Resets the elapsed real-time of this clock since system boot, in nanoseconds.
+     * @hide
+     */
+    @TestApi
+    public void resetElapsedRealtimeNanos() {
+        resetFlag(HAS_ELAPSED_REALTIME_NANOS);
+        mElapsedRealtimeNanos = 0;
+    }
+
+    /**
+     * Returns {@code true} if {@link #getElapsedRealtimeUncertaintyNanos()} is available, {@code
+     * false} otherwise.
+     */
+    public boolean hasElapsedRealtimeUncertaintyNanos() {
+        return isFlagSet(HAS_ELAPSED_REALTIME_UNCERTAINTY_NANOS);
+    }
+
+    /**
+     * Gets the estimate of the relative precision of the alignment of the
+     * {@link #getElapsedRealtimeNanos()} timestamp, with the reported measurements in
+     * nanoseconds (68% confidence).
+     *
+     * <p>The value is only available if {@link #hasElapsedRealtimeUncertaintyNanos()} is
+     * {@code true}.
+     */
+    @FloatRange(from = 0.0f)
+    public double getElapsedRealtimeUncertaintyNanos() {
+        return mElapsedRealtimeUncertaintyNanos;
+    }
+
+    /**
+     * Sets the estimate of the relative precision of the alignment of the
+     * {@link #getElapsedRealtimeNanos()} timestamp, with the reported measurements in
+     * nanoseconds (68% confidence).
+     * @hide
+     */
+    @TestApi
+    public void setElapsedRealtimeUncertaintyNanos(
+            @FloatRange(from = 0.0f) double elapsedRealtimeUncertaintyNanos) {
+        setFlag(HAS_ELAPSED_REALTIME_UNCERTAINTY_NANOS);
+        mElapsedRealtimeUncertaintyNanos = elapsedRealtimeUncertaintyNanos;
+    }
+
+    /**
+     * Resets the estimate of the relative precision of the alignment of the
+     * {@link #getElapsedRealtimeNanos()} timestamp, with the reported measurements in
+     * nanoseconds (68% confidence).
+     * @hide
+     */
+    @TestApi
+    public void resetElapsedRealtimeUncertaintyNanos() {
+        resetFlag(HAS_ELAPSED_REALTIME_UNCERTAINTY_NANOS);
+        mElapsedRealtimeUncertaintyNanos = Double.NaN;
+    }
+
     /**
      * Gets count of hardware clock discontinuities.
      *
@@ -428,7 +526,7 @@ public final class GnssClock implements Parcelable {
         mHardwareClockDiscontinuityCount = value;
     }
 
-    public static final Creator<GnssClock> CREATOR = new Creator<GnssClock>() {
+    public static final @android.annotation.NonNull Creator<GnssClock> CREATOR = new Creator<GnssClock>() {
         @Override
         public GnssClock createFromParcel(Parcel parcel) {
             GnssClock gpsClock = new GnssClock();
@@ -443,6 +541,8 @@ public final class GnssClock implements Parcelable {
             gpsClock.mDriftNanosPerSecond = parcel.readDouble();
             gpsClock.mDriftUncertaintyNanosPerSecond = parcel.readDouble();
             gpsClock.mHardwareClockDiscontinuityCount = parcel.readInt();
+            gpsClock.mElapsedRealtimeNanos = parcel.readLong();
+            gpsClock.mElapsedRealtimeUncertaintyNanos = parcel.readDouble();
 
             return gpsClock;
         }
@@ -465,6 +565,8 @@ public final class GnssClock implements Parcelable {
         parcel.writeDouble(mDriftNanosPerSecond);
         parcel.writeDouble(mDriftUncertaintyNanosPerSecond);
         parcel.writeInt(mHardwareClockDiscontinuityCount);
+        parcel.writeLong(mElapsedRealtimeNanos);
+        parcel.writeDouble(mElapsedRealtimeUncertaintyNanos);
     }
 
     @Override
@@ -511,6 +613,16 @@ public final class GnssClock implements Parcelable {
                 "HardwareClockDiscontinuityCount",
                 mHardwareClockDiscontinuityCount));
 
+        builder.append(String.format(
+                format,
+                "ElapsedRealtimeNanos",
+                hasElapsedRealtimeNanos() ? mElapsedRealtimeNanos : null));
+
+        builder.append(String.format(
+                format,
+                "ElapsedRealtimeUncertaintyNanos",
+                hasElapsedRealtimeUncertaintyNanos() ? mElapsedRealtimeUncertaintyNanos : null));
+
         return builder.toString();
     }
 
@@ -525,6 +637,8 @@ public final class GnssClock implements Parcelable {
         resetDriftNanosPerSecond();
         resetDriftUncertaintyNanosPerSecond();
         setHardwareClockDiscontinuityCount(Integer.MIN_VALUE);
+        resetElapsedRealtimeNanos();
+        resetElapsedRealtimeUncertaintyNanos();
     }
 
     private void setFlag(int flag) {
