@@ -190,7 +190,7 @@ public final class MidiDeviceInfo implements Parcelable {
     }
 
     private final int mType;    // USB or virtual
-    private final int mId;      // unique ID generated by MidiService
+    private final int mId;      // unique ID generated by MidiService. Accessed from native code.
     private final int mInputPortCount;
     private final int mOutputPortCount;
     private final String[] mInputPortNames;
@@ -325,7 +325,7 @@ public final class MidiDeviceInfo implements Parcelable {
                 ",mIsPrivate=" + mIsPrivate);
     }
 
-    public static final Parcelable.Creator<MidiDeviceInfo> CREATOR =
+    public static final @android.annotation.NonNull Parcelable.Creator<MidiDeviceInfo> CREATOR =
         new Parcelable.Creator<MidiDeviceInfo>() {
         public MidiDeviceInfo createFromParcel(Parcel in) {
             // Needs to be kept in sync with code in MidiDeviceInfo.cpp
