@@ -16,7 +16,15 @@
 
 package android.os;
 
+import android.annotation.SystemApi;
+import android.annotation.TestApi;
+
 /** @hide */
+@SystemApi
+@TestApi
 public interface IHwInterface {
+    /**
+     * @return the binder object that corresponds to this interface.
+     */
     public IHwBinder asBinder();
 }
