@@ -23,11 +23,13 @@ import android.annotation.NonNull;
 import android.annotation.Nullable;
 import android.annotation.StringRes;
 import android.annotation.TestApi;
+import android.annotation.UnsupportedAppUsage;
 import android.content.pm.PackageManager;
 import android.content.res.Resources;
 import android.os.Bundle;
 import android.os.Parcel;
 import android.os.Parcelable;
+import android.service.print.PrintJobInfoProto;
 
 import com.android.internal.util.Preconditions;
 
@@ -44,9 +46,14 @@ import java.util.Arrays;
 public final class PrintJobInfo implements Parcelable {
 
     /** @hide */
-    @IntDef({
-            STATE_CREATED, STATE_QUEUED, STATE_STARTED, STATE_BLOCKED, STATE_COMPLETED,
-            STATE_FAILED, STATE_CANCELED
+    @IntDef(prefix = { "STATE_" }, value = {
+            STATE_CREATED,
+            STATE_QUEUED,
+            STATE_STARTED,
+            STATE_BLOCKED,
+            STATE_COMPLETED,
+            STATE_FAILED,
+            STATE_CANCELED
     })
     @Retention(RetentionPolicy.SOURCE)
     public @interface State {
@@ -88,7 +95,7 @@ public final class PrintJobInfo implements Parcelable {
      * Next valid states: {@link #STATE_QUEUED}
      * </p>
      */
-    public static final int STATE_CREATED = 1;
+    public static final int STATE_CREATED = PrintJobInfoProto.STATE_CREATED;
 
     /**
      * Print job state: The print jobs is created, it is ready
@@ -98,7 +105,7 @@ public final class PrintJobInfo implements Parcelable {
      * {@link #STATE_CANCELED}
      * </p>
      */
-    public static final int STATE_QUEUED = 2;
+    public static final int STATE_QUEUED = PrintJobInfoProto.STATE_QUEUED;
 
     /**
      * Print job state: The print job is being printed.
@@ -107,7 +114,7 @@ public final class PrintJobInfo implements Parcelable {
      * {@link #STATE_CANCELED}, {@link #STATE_BLOCKED}
      * </p>
      */
-    public static final int STATE_STARTED = 3;
+    public static final int STATE_STARTED = PrintJobInfoProto.STATE_STARTED;
 
     /**
      * Print job state: The print job is blocked.
@@ -116,7 +123,7 @@ public final class PrintJobInfo implements Parcelable {
      * {@link #STATE_STARTED}
      * </p>
      */
-    public static final int STATE_BLOCKED = 4;
+    public static final int STATE_BLOCKED = PrintJobInfoProto.STATE_BLOCKED;
 
     /**
      * Print job state: The print job is successfully printed.
@@ -125,7 +132,7 @@ public final class PrintJobInfo implements Parcelable {
      * Next valid states: None
      * </p>
      */
-    public static final int STATE_COMPLETED = 5;
+    public static final int STATE_COMPLETED = PrintJobInfoProto.STATE_COMPLETED;
 
     /**
      * Print job state: The print job was printing but printing failed.
@@ -133,7 +140,7 @@ public final class PrintJobInfo implements Parcelable {
      * Next valid states: {@link #STATE_CANCELED}, {@link #STATE_STARTED}
      * </p>
      */
-    public static final int STATE_FAILED = 6;
+    public static final int STATE_FAILED = PrintJobInfoProto.STATE_FAILED;
 
     /**
      * Print job state: The print job is canceled.
@@ -142,7 +149,7 @@ public final class PrintJobInfo implements Parcelable {
      * Next valid states: None
      * </p>
      */
-    public static final int STATE_CANCELED = 7;
+    public static final int STATE_CANCELED = PrintJobInfoProto.STATE_CANCELED;
 
     /** The unique print job id. */
     private PrintJobId mId;
@@ -543,6 +550,7 @@ public final class PrintJobInfo implements Parcelable {
      *
      * @hide
      */
+    @UnsupportedAppUsage
     public PrintDocumentInfo getDocumentInfo() {
         return mDocumentInfo;
     }
@@ -580,6 +588,15 @@ public final class PrintJobInfo implements Parcelable {
         mCanceling = cancelling;
     }
 
+    /**
+     * If the print job is actively processed, i.e. the device needs to stay on.
+     *
+     * @hide
+     */
+    public boolean shouldStayAwake() {
+        return mCanceling || mState == STATE_STARTED || mState == STATE_QUEUED;
+    }
+
     /**
      * Gets whether this job has a given advanced (printer specific) print
      * option.
@@ -624,6 +641,7 @@ public final class PrintJobInfo implements Parcelable {
      *
      * @hide
      */
+    @UnsupportedAppUsage
     public Bundle getAdvancedOptions() {
         return mAdvancedOptions;
     }
@@ -863,7 +881,7 @@ public final class PrintJobInfo implements Parcelable {
         }
     }
 
-    public static final Parcelable.Creator<PrintJobInfo> CREATOR =
+    public static final @android.annotation.NonNull Parcelable.Creator<PrintJobInfo> CREATOR =
             new Creator<PrintJobInfo>() {
         @Override
         public PrintJobInfo createFromParcel(Parcel parcel) {
