@@ -31,6 +31,7 @@ import android.graphics.drawable.Drawable;
 import android.graphics.drawable.Icon;
 import android.os.Parcel;
 import android.os.Parcelable;
+import android.service.print.PrinterInfoProto;
 import android.text.TextUtils;
 
 import com.android.internal.util.Preconditions;
@@ -52,20 +53,23 @@ import java.lang.annotation.RetentionPolicy;
 public final class PrinterInfo implements Parcelable {
 
     /** @hide */
-    @IntDef({
-            STATUS_IDLE, STATUS_BUSY, STATUS_UNAVAILABLE
+    @IntDef(prefix = { "STATUS_" }, value = {
+            STATUS_IDLE,
+            STATUS_BUSY,
+            STATUS_UNAVAILABLE
     })
     @Retention(RetentionPolicy.SOURCE)
     public @interface Status {
     }
+
     /** Printer status: the printer is idle and ready to print. */
-    public static final int STATUS_IDLE = 1;
+    public static final int STATUS_IDLE = PrinterInfoProto.STATUS_IDLE;
 
     /** Printer status: the printer is busy printing. */
-    public static final int STATUS_BUSY = 2;
+    public static final int STATUS_BUSY = PrinterInfoProto.STATUS_BUSY;
 
     /** Printer status: the printer is not available. */
-    public static final int STATUS_UNAVAILABLE = 3;
+    public static final int STATUS_UNAVAILABLE = PrinterInfoProto.STATUS_UNAVAILABLE;
 
     private final @NonNull PrinterId mId;
 
@@ -559,7 +563,7 @@ public final class PrinterInfo implements Parcelable {
         }
     }
 
-    public static final Parcelable.Creator<PrinterInfo> CREATOR =
+    public static final @android.annotation.NonNull Parcelable.Creator<PrinterInfo> CREATOR =
             new Parcelable.Creator<PrinterInfo>() {
         @Override
         public PrinterInfo createFromParcel(Parcel parcel) {
