@@ -32,6 +32,7 @@ import android.os.IBinder;
 import android.os.Looper;
 import android.os.Message;
 import android.os.RemoteException;
+import android.util.Log;
 import android.view.View;
 import android.view.View.OnAttachStateChangeListener;
 import android.view.WindowManager;
@@ -79,6 +80,9 @@ import com.android.internal.R;
  */
 public class TileService extends Service {
 
+    private static final String TAG = "TileService";
+    private static final boolean DEBUG = false;
+
     /**
      * An activity that provides a user interface for adjusting TileService
      * preferences. Optional but recommended for apps that implement a
@@ -87,6 +91,10 @@ public class TileService extends Service {
      * This intent may also define a {@link Intent#EXTRA_COMPONENT_NAME} value
      * to indicate the {@link ComponentName} that caused the preferences to be
      * opened.
+     * <p>
+     * To ensure that the activity can only be launched through quick settings
+     * UI provided by this service, apps can protect it with the
+     * BIND_QUICK_SETTINGS_TILE permission.
      */
     @SdkConstant(SdkConstantType.INTENT_CATEGORY)
     public static final String ACTION_QS_TILE_PREFERENCES
@@ -377,18 +385,26 @@ public class TileService extends Service {
         private static final int MSG_TILE_CLICKED = 5;
         private static final int MSG_UNLOCK_COMPLETE = 6;
         private static final int MSG_START_SUCCESS = 7;
+        private final String mTileServiceName;
 
         public H(Looper looper) {
             super(looper);
+            mTileServiceName = TileService.this.getClass().getSimpleName();
+        }
+
+        private void logMessage(String message) {
+            Log.d(TAG, mTileServiceName + " Handler - " + message);
         }
 
         @Override
         public void handleMessage(Message msg) {
             switch (msg.what) {
                 case MSG_TILE_ADDED:
+                    if (DEBUG) logMessage("MSG_TILE_ADDED");
                     TileService.this.onTileAdded();
                     break;
                 case MSG_TILE_REMOVED:
+                    if (DEBUG) logMessage("MSG_TILE_REMOVED");
                     if (mListening) {
                         mListening = false;
                         TileService.this.onStopListening();
@@ -396,27 +412,32 @@ public class TileService extends Service {
                     TileService.this.onTileRemoved();
                     break;
                 case MSG_STOP_LISTENING:
+                    if (DEBUG) logMessage("MSG_STOP_LISTENING");
                     if (mListening) {
                         mListening = false;
                         TileService.this.onStopListening();
                     }
                     break;
                 case MSG_START_LISTENING:
+                    if (DEBUG) logMessage("MSG_START_LISTENING");
                     if (!mListening) {
                         mListening = true;
                         TileService.this.onStartListening();
                     }
                     break;
                 case MSG_TILE_CLICKED:
+                    if (DEBUG) logMessage("MSG_TILE_CLICKED");
                     mToken = (IBinder) msg.obj;
                     TileService.this.onClick();
                     break;
                 case MSG_UNLOCK_COMPLETE:
+                    if (DEBUG) logMessage("MSG_UNLOCK_COMPLETE");
                     if (mUnlockRunnable != null) {
                         mUnlockRunnable.run();
                     }
                     break;
                 case MSG_START_SUCCESS:
+                    if (DEBUG) logMessage("MSG_START_SUCCESS");
                     try {
                         mService.onStartSuccessful(mTileToken);
                     } catch (RemoteException e) {
