@@ -19,6 +19,7 @@ package android.service.voice;
 import android.annotation.IntDef;
 import android.annotation.NonNull;
 import android.annotation.Nullable;
+import android.annotation.UnsupportedAppUsage;
 import android.app.Activity;
 import android.content.Intent;
 import android.hardware.soundtrigger.IRecognitionStatusCallback;
@@ -87,11 +88,11 @@ public class AlwaysOnHotwordDetector {
 
     // Keyphrase management actions. Used in getManageIntent() ----//
     @Retention(RetentionPolicy.SOURCE)
-    @IntDef(value = {
-                MANAGE_ACTION_ENROLL,
-                MANAGE_ACTION_RE_ENROLL,
-                MANAGE_ACTION_UN_ENROLL
-            })
+    @IntDef(prefix = { "MANAGE_ACTION_" }, value = {
+            MANAGE_ACTION_ENROLL,
+            MANAGE_ACTION_RE_ENROLL,
+            MANAGE_ACTION_UN_ENROLL
+    })
     private @interface ManageActions {}
 
     /**
@@ -116,12 +117,11 @@ public class AlwaysOnHotwordDetector {
     //-- Flags for startRecognition    ----//
     /** @hide */
     @Retention(RetentionPolicy.SOURCE)
-    @IntDef(flag = true,
-            value = {
-                RECOGNITION_FLAG_NONE,
-                RECOGNITION_FLAG_CAPTURE_TRIGGER_AUDIO,
-                RECOGNITION_FLAG_ALLOW_MULTIPLE_TRIGGERS
-            })
+    @IntDef(flag = true, prefix = { "RECOGNITION_FLAG_" }, value = {
+            RECOGNITION_FLAG_NONE,
+            RECOGNITION_FLAG_CAPTURE_TRIGGER_AUDIO,
+            RECOGNITION_FLAG_ALLOW_MULTIPLE_TRIGGERS
+    })
     public @interface RecognitionFlags {}
 
     /**
@@ -150,11 +150,10 @@ public class AlwaysOnHotwordDetector {
 
     /** @hide */
     @Retention(RetentionPolicy.SOURCE)
-    @IntDef(flag = true,
-            value = {
-                RECOGNITION_MODE_VOICE_TRIGGER,
-                RECOGNITION_MODE_USER_IDENTIFICATION,
-            })
+    @IntDef(flag = true, prefix = { "RECOGNITION_MODE_" }, value = {
+            RECOGNITION_MODE_VOICE_TRIGGER,
+            RECOGNITION_MODE_USER_IDENTIFICATION,
+    })
     public @interface RecognitionModes {}
 
     /**
@@ -268,6 +267,7 @@ public class AlwaysOnHotwordDetector {
          * @hide
          */
         @Nullable
+        @UnsupportedAppUsage
         public Integer getCaptureSession() {
             if (mCaptureAvailable) {
                 return mCaptureSession;
