@@ -17,6 +17,7 @@
 package android.transition;
 
 import android.annotation.TestApi;
+import android.annotation.UnsupportedAppUsage;
 import android.content.Context;
 import android.util.ArrayMap;
 import android.util.Log;
@@ -73,9 +74,11 @@ public class TransitionManager {
     ArrayMap<Scene, Transition> mSceneTransitions = new ArrayMap<Scene, Transition>();
     ArrayMap<Scene, ArrayMap<Scene, Transition>> mScenePairTransitions =
             new ArrayMap<Scene, ArrayMap<Scene, Transition>>();
+    @UnsupportedAppUsage
     private static ThreadLocal<WeakReference<ArrayMap<ViewGroup, ArrayList<Transition>>>>
             sRunningTransitions =
             new ThreadLocal<WeakReference<ArrayMap<ViewGroup, ArrayList<Transition>>>>();
+    @UnsupportedAppUsage
     private static ArrayList<ViewGroup> sPendingTransitions = new ArrayList<ViewGroup>();
 
 
@@ -185,7 +188,13 @@ public class TransitionManager {
 
         final ViewGroup sceneRoot = scene.getSceneRoot();
         if (!sPendingTransitions.contains(sceneRoot)) {
+            Scene oldScene = Scene.getCurrentScene(sceneRoot);
             if (transition == null) {
+                // Notify old scene that it is being exited
+                if (oldScene != null) {
+                    oldScene.exit();
+                }
+
                 scene.enter();
             } else {
                 sPendingTransitions.add(sceneRoot);
@@ -193,7 +202,6 @@ public class TransitionManager {
                 Transition transitionClone = transition.clone();
                 transitionClone.setSceneRoot(sceneRoot);
 
-                Scene oldScene = Scene.getCurrentScene(sceneRoot);
                 if (oldScene != null && oldScene.isCreatedFromLayoutResource()) {
                     transitionClone.setCanRemoveViews(true);
                 }
@@ -207,17 +215,20 @@ public class TransitionManager {
         }
     }
 
+    @UnsupportedAppUsage
     private static ArrayMap<ViewGroup, ArrayList<Transition>> getRunningTransitions() {
         WeakReference<ArrayMap<ViewGroup, ArrayList<Transition>>> runningTransitions =
                 sRunningTransitions.get();
-        if (runningTransitions == null || runningTransitions.get() == null) {
-            ArrayMap<ViewGroup, ArrayList<Transition>> transitions =
-                    new ArrayMap<ViewGroup, ArrayList<Transition>>();
-            runningTransitions = new WeakReference<ArrayMap<ViewGroup, ArrayList<Transition>>>(
-                    transitions);
-            sRunningTransitions.set(runningTransitions);
+        if (runningTransitions != null) {
+            ArrayMap<ViewGroup, ArrayList<Transition>> transitions = runningTransitions.get();
+            if (transitions != null) {
+                return transitions;
+            }
         }
-        return runningTransitions.get();
+        ArrayMap<ViewGroup, ArrayList<Transition>> transitions = new ArrayMap<>();
+        runningTransitions = new WeakReference<>(transitions);
+        sRunningTransitions.set(runningTransitions);
+        return transitions;
     }
 
     private static void sceneChangeRunTransition(final ViewGroup sceneRoot,
@@ -304,6 +315,7 @@ public class TransitionManager {
                     ArrayList<Transition> currentTransitions =
                             runningTransitions.get(mSceneRoot);
                     currentTransitions.remove(transition);
+                    transition.removeListener(this);
                 }
             });
             mTransition.captureValues(mSceneRoot, false);
