@@ -451,5 +451,10 @@ public interface Menu {
      *                 will use numeric shortcuts.
      */
     public void setQwertyMode(boolean isQwerty);
-}
 
+    /**
+     * Enable or disable the group dividers.
+     */
+    default void setGroupDividerEnabled(boolean groupDividerEnabled) {
+    }
+}
\ No newline at end of file
