@@ -23,7 +23,7 @@ import android.content.ContextWrapper;
 import android.content.res.ColorStateList;
 import android.content.res.TypedArray;
 import android.content.res.XmlResourceParser;
-import android.graphics.PorterDuff;
+import android.graphics.BlendMode;
 import android.graphics.drawable.Drawable;
 import android.util.AttributeSet;
 import android.util.Log;
@@ -308,7 +308,7 @@ public class MenuInflater {
         private CharSequence itemTitleCondensed;
         private int itemIconResId;
         private ColorStateList itemIconTintList = null;
-        private PorterDuff.Mode itemIconTintMode = null;
+        private BlendMode mItemIconBlendMode = null;
         private char itemAlphabeticShortcut;
         private int itemAlphabeticModifiers;
         private char itemNumericShortcut;
@@ -401,12 +401,12 @@ public class MenuInflater {
             itemTitleCondensed = a.getText(com.android.internal.R.styleable.MenuItem_titleCondensed);
             itemIconResId = a.getResourceId(com.android.internal.R.styleable.MenuItem_icon, 0);
             if (a.hasValue(com.android.internal.R.styleable.MenuItem_iconTintMode)) {
-                itemIconTintMode = Drawable.parseTintMode(a.getInt(
+                mItemIconBlendMode = Drawable.parseBlendMode(a.getInt(
                         com.android.internal.R.styleable.MenuItem_iconTintMode, -1),
-                        itemIconTintMode);
+                        mItemIconBlendMode);
             } else {
                 // Reset to null so that it's not carried over to the next item
-                itemIconTintMode = null;
+                mItemIconBlendMode = null;
             }
             if (a.hasValue(com.android.internal.R.styleable.MenuItem_iconTint)) {
                 itemIconTintList = a.getColorStateList(
@@ -487,8 +487,8 @@ public class MenuInflater {
                 item.setShowAsAction(itemShowAsAction);
             }
 
-            if (itemIconTintMode != null) {
-                item.setIconTintMode(itemIconTintMode);
+            if (mItemIconBlendMode != null) {
+                item.setIconTintBlendMode(mItemIconBlendMode);
             }
 
             if (itemIconTintList != null) {
