@@ -17,7 +17,9 @@
 package android.webkit;
 
 import android.annotation.IntDef;
+import android.annotation.Nullable;
 import android.annotation.SystemApi;
+import android.annotation.UnsupportedAppUsage;
 import android.content.Context;
 
 import java.lang.annotation.ElementType;
@@ -28,10 +30,10 @@ import java.lang.annotation.Target;
 /**
  * Manages settings state for a WebView. When a WebView is first created, it
  * obtains a set of default settings. These default settings will be returned
- * from any getter call. A WebSettings object obtained from
- * WebView.getSettings() is tied to the life of the WebView. If a WebView has
- * been destroyed, any method call on WebSettings will throw an
- * IllegalStateException.
+ * from any getter call. A {@code WebSettings} object obtained from
+ * {@link WebView#getSettings()} is tied to the life of the WebView. If a WebView has
+ * been destroyed, any method call on {@code WebSettings} will throw an
+ * {@link IllegalStateException}.
  */
 // This is an abstract base class: concrete WebViewProviders must
 // create a class derived from this, and return an instance of it in the
@@ -40,13 +42,13 @@ public abstract class WebSettings {
     /**
      * Enum for controlling the layout of html.
      * <ul>
-     *   <li>NORMAL means no rendering changes. This is the recommended choice for maximum
+     *   <li>{@code NORMAL} means no rendering changes. This is the recommended choice for maximum
      *       compatibility across different platforms and Android versions.</li>
-     *   <li>SINGLE_COLUMN moves all content into one column that is the width of the
+     *   <li>{@code SINGLE_COLUMN} moves all content into one column that is the width of the
      *       view.</li>
-     *   <li>NARROW_COLUMNS makes all columns no wider than the screen if possible. Only use
+     *   <li>{@code NARROW_COLUMNS} makes all columns no wider than the screen if possible. Only use
      *       this for API levels prior to {@link android.os.Build.VERSION_CODES#KITKAT}.</li>
-     *   <li>TEXT_AUTOSIZING boosts font size of paragraphs based on heuristics to make
+     *   <li>{@code TEXT_AUTOSIZING} boosts font size of paragraphs based on heuristics to make
      *       the text readable when viewing a wide-viewport layout in the overview mode.
      *       It is recommended to enable zoom support {@link #setSupportZoom} when
      *       using this mode. Supported from API level
@@ -91,15 +93,16 @@ public abstract class WebSettings {
         TextSize(int size) {
             value = size;
         }
+        @UnsupportedAppUsage
         int value;
     }
 
     /**
      * Enum for specifying the WebView's desired density.
      * <ul>
-     *   <li>FAR makes 100% looking like in 240dpi</li>
-     *   <li>MEDIUM makes 100% looking like in 160dpi</li>
-     *   <li>CLOSE makes 100% looking like in 120dpi</li>
+     *   <li>{@code FAR} makes 100% looking like in 240dpi</li>
+     *   <li>{@code MEDIUM} makes 100% looking like in 160dpi</li>
+     *   <li>{@code CLOSE} makes 100% looking like in 120dpi</li>
      * </ul>
      */
     public enum ZoomDensity {
@@ -121,7 +124,13 @@ public abstract class WebSettings {
     }
 
     /** @hide */
-    @IntDef({LOAD_DEFAULT, LOAD_NORMAL, LOAD_CACHE_ELSE_NETWORK, LOAD_NO_CACHE, LOAD_CACHE_ONLY})
+    @IntDef(prefix = { "LOAD_" }, value = {
+            LOAD_DEFAULT,
+            LOAD_NORMAL,
+            LOAD_CACHE_ELSE_NETWORK,
+            LOAD_NO_CACHE,
+            LOAD_CACHE_ONLY
+    })
     @Retention(RetentionPolicy.SOURCE)
     public @interface CacheMode {}
 
@@ -184,26 +193,24 @@ public abstract class WebSettings {
     }
 
     /**
-     * Used with {@link #setMixedContentMode}
-     *
      * In this mode, the WebView will allow a secure origin to load content from any other origin,
      * even if that origin is insecure. This is the least secure mode of operation for the WebView,
      * and where possible apps should not set this mode.
+     *
+     * @see #setMixedContentMode
      */
     public static final int MIXED_CONTENT_ALWAYS_ALLOW = 0;
 
     /**
-     * Used with {@link #setMixedContentMode}
-     *
      * In this mode, the WebView will not allow a secure origin to load content from an insecure
      * origin. This is the preferred and most secure mode of operation for the WebView and apps are
      * strongly advised to use this mode.
+     *
+     * @see #setMixedContentMode
      */
     public static final int MIXED_CONTENT_NEVER_ALLOW = 1;
 
     /**
-     * Used with {@link #setMixedContentMode}
-     *
      * In this mode, the WebView will attempt to be compatible with the approach of a modern web
      * browser with regard to mixed content. Some insecure content may be allowed to be loaded by
      * a secure origin and other types of content will be blocked. The types of content are allowed
@@ -212,12 +219,51 @@ public abstract class WebSettings {
      * This mode is intended to be used by apps that are not in control of the content that they
      * render but desire to operate in a reasonably secure environment. For highest security, apps
      * are recommended to use {@link #MIXED_CONTENT_NEVER_ALLOW}.
+     *
+     * @see #setMixedContentMode
      */
     public static final int MIXED_CONTENT_COMPATIBILITY_MODE = 2;
 
+    /** @hide */
+    @IntDef(prefix = { "FORCE_DARK_" }, value = {
+            FORCE_DARK_OFF,
+            FORCE_DARK_AUTO,
+            FORCE_DARK_ON
+    })
+    @Retention(RetentionPolicy.SOURCE)
+    public @interface ForceDark {}
+
+    /**
+     * Disable force dark, irrespective of the force dark mode of the WebView parent. In this mode,
+     * WebView content will always be rendered as-is, regardless of whether native views are being
+     * automatically darkened.
+     *
+     * @see #setForceDark
+     */
+    public static final int FORCE_DARK_OFF = 0;
+
+    /**
+     * Enable force dark dependent on the state of the WebView parent view. If the WebView parent
+     * view is being automatically force darkened
+     * (see: {@link android.view.View#setForceDarkAllowed}), then WebView content will be rendered
+     * so as to emulate a dark theme. WebViews that are not attached to the view hierarchy will not
+     * be inverted.
+     *
+     * @see #setForceDark
+     */
+    public static final int FORCE_DARK_AUTO = 1;
+
+    /**
+     * Unconditionally enable force dark. In this mode WebView content will always be rendered so
+     * as to emulate a dark theme.
+     *
+     * @see #setForceDark
+     */
+    public static final int FORCE_DARK_ON = 2;
+
     /**
      * Enables dumping the pages navigation cache to a text file. The default
-     * is false.
+     * is {@code false}.
      *
      * @deprecated This method is now obsolete.
      * @hide Since API level {@link android.os.Build.VERSION_CODES#JELLY_BEAN_MR1}
@@ -243,7 +289,7 @@ public abstract class WebSettings {
      * controls and gestures. The particular zoom mechanisms that should be used
      * can be set with {@link #setBuiltInZoomControls}. This setting does not
      * affect zooming performed using the {@link WebView#zoomIn()} and
-     * {@link WebView#zoomOut()} methods. The default is true.
+     * {@link WebView#zoomOut()} methods. The default is {@code true}.
      *
      * @param support whether the WebView should support zoom
      */
@@ -252,14 +298,14 @@ public abstract class WebSettings {
     /**
      * Gets whether the WebView supports zoom.
      *
-     * @return true if the WebView supports zoom
+     * @return {@code true} if the WebView supports zoom
      * @see #setSupportZoom
      */
     public abstract boolean supportZoom();
 
     /**
      * Sets whether the WebView requires a user gesture to play media.
-     * The default is true.
+     * The default is {@code true}.
      *
      * @param require whether the WebView requires a user gesture to play media
      */
@@ -268,7 +314,7 @@ public abstract class WebSettings {
     /**
      * Gets whether the WebView requires a user gesture to play media.
      *
-     * @return true if the WebView requires a user gesture to play media
+     * @return {@code true} if the WebView requires a user gesture to play media
      * @see #setMediaPlaybackRequiresUserGesture
      */
     public abstract boolean getMediaPlaybackRequiresUserGesture();
@@ -278,10 +324,13 @@ public abstract class WebSettings {
      * built-in zoom mechanisms comprise on-screen zoom controls, which are
      * displayed over the WebView's content, and the use of a pinch gesture to
      * control zooming. Whether or not these on-screen controls are displayed
-     * can be set with {@link #setDisplayZoomControls}. The default is false.
+     * can be set with {@link #setDisplayZoomControls}. The default is {@code false}.
      * <p>
      * The built-in mechanisms are the only currently supported zoom
      * mechanisms, so it is recommended that this setting is always enabled.
+     * However, on-screen zoom controls are deprecated in Android (see
+     * {@link android.widget.ZoomButtonsController}) so it's recommended to
+     * disable {@link #setDisplayZoomControls}.
      *
      * @param enabled whether the WebView should use its built-in zoom mechanisms
      */
@@ -293,7 +342,7 @@ public abstract class WebSettings {
     /**
      * Gets whether the zoom mechanisms built into WebView are being used.
      *
-     * @return true if the zoom mechanisms built into WebView are being used
+     * @return {@code true} if the zoom mechanisms built into WebView are being used
      * @see #setBuiltInZoomControls
      */
     public abstract boolean getBuiltInZoomControls();
@@ -301,7 +350,9 @@ public abstract class WebSettings {
     /**
      * Sets whether the WebView should display on-screen zoom controls when
      * using the built-in zoom mechanisms. See {@link #setBuiltInZoomControls}.
-     * The default is true.
+     * The default is {@code true}. However, on-screen zoom controls are deprecated
+     * in Android (see {@link android.widget.ZoomButtonsController}) so it's
+     * recommended to set this to {@code false}.
      *
      * @param enabled whether the WebView should display on-screen zoom controls
      */
@@ -311,7 +362,7 @@ public abstract class WebSettings {
      * Gets whether the WebView displays on-screen zoom controls when using
      * the built-in zoom mechanisms.
      *
-     * @return true if the WebView displays on-screen zoom controls when using
+     * @return {@code true} if the WebView displays on-screen zoom controls when using
      *         the built-in zoom mechanisms
      * @see #setDisplayZoomControls
      */
@@ -351,7 +402,7 @@ public abstract class WebSettings {
      * zooms out the content to fit on screen by width. This setting is
      * taken into account when the content width is greater than the width
      * of the WebView control, for example, when {@link #getUseWideViewPort}
-     * is enabled. The default is false.
+     * is enabled. The default is {@code false}.
      */
     public abstract void setLoadWithOverviewMode(boolean overview);
 
@@ -366,9 +417,9 @@ public abstract class WebSettings {
     /**
      * Sets whether the WebView will enable smooth transition while panning or
      * zooming or while the window hosting the WebView does not have focus.
-     * If it is true, WebView will choose a solution to maximize the performance.
+     * If it is {@code true}, WebView will choose a solution to maximize the performance.
      * e.g. the WebView's content may not be updated during the transition.
-     * If it is false, WebView will keep its fidelity. The default value is false.
+     * If it is false, WebView will keep its fidelity. The default value is {@code false}.
      *
      * @deprecated This method is now obsolete, and will become a no-op in future.
      */
@@ -388,8 +439,8 @@ public abstract class WebSettings {
 
     /**
      * Sets whether the WebView uses its background for over scroll background.
-     * If true, it will use the WebView's background. If false, it will use an
-     * internal pattern. Default is true.
+     * If {@code true}, it will use the WebView's background. If {@code false}, it will use an
+     * internal pattern. Default is {@code true}.
      *
      * @deprecated This method is now obsolete.
      * @hide Since API level {@link android.os.Build.VERSION_CODES#JELLY_BEAN_MR1}
@@ -418,7 +469,9 @@ public abstract class WebSettings {
      * Note that the feature will continue to be supported on older versions of
      * Android as before.
      *
-     * This function does not have any effect.
+     * @deprecated In Android O and afterwards, this function does not have
+     * any effect, the form data will be saved to platform's autofill service
+     * if applicable.
      */
     @Deprecated
     public abstract  void setSaveFormData(boolean save);
@@ -433,7 +486,7 @@ public abstract class WebSettings {
     public abstract boolean getSaveFormData();
 
     /**
-     * Sets whether the WebView should save passwords. The default is true.
+     * Sets whether the WebView should save passwords. The default is {@code true}.
      * @deprecated Saving passwords in WebView will not be supported in future versions.
      */
     @Deprecated
@@ -572,6 +625,7 @@ public abstract class WebSettings {
      * @hide Since API level {@link android.os.Build.VERSION_CODES#JELLY_BEAN_MR1}
      */
     @Deprecated
+    @UnsupportedAppUsage
     public void setUseDoubleTree(boolean use) {
         // Specified to do nothing, so no need for derived classes to override.
     }
@@ -584,6 +638,7 @@ public abstract class WebSettings {
      * @hide Since API level {@link android.os.Build.VERSION_CODES#JELLY_BEAN_MR1}
      */
     @Deprecated
+    @UnsupportedAppUsage
     public boolean getUseDoubleTree() {
         // Returns false unconditionally, so no need for derived classes to override.
         return false;
@@ -627,9 +682,9 @@ public abstract class WebSettings {
     /**
      * Sets whether the WebView should enable support for the &quot;viewport&quot;
      * HTML meta tag or should use a wide viewport.
-     * When the value of the setting is false, the layout width is always set to the
+     * When the value of the setting is {@code false}, the layout width is always set to the
      * width of the WebView control in device-independent (CSS) pixels.
-     * When the value is true and the page contains the viewport meta tag, the value
+     * When the value is {@code true} and the page contains the viewport meta tag, the value
      * of the width specified in the tag is used. If the page does not contain the tag or
      * does not provide a width, then a wide viewport will be used.
      *
@@ -641,7 +696,7 @@ public abstract class WebSettings {
      * Gets whether the WebView supports the &quot;viewport&quot;
      * HTML meta tag or will use a wide viewport.
      *
-     * @return true if the WebView supports the viewport meta tag
+     * @return {@code true} if the WebView supports the viewport meta tag
      * @see #setUseWideViewPort
      */
     public abstract boolean getUseWideViewPort();
@@ -649,22 +704,22 @@ public abstract class WebSettings {
     /**
      * Sets whether the WebView whether supports multiple windows. If set to
      * true, {@link WebChromeClient#onCreateWindow} must be implemented by the
-     * host application. The default is false.
+     * host application. The default is {@code false}.
      *
-     * @param support whether to suport multiple windows
+     * @param support whether to support multiple windows
      */
     public abstract void setSupportMultipleWindows(boolean support);
 
     /**
      * Gets whether the WebView supports multiple windows.
      *
-     * @return true if the WebView supports multiple windows
+     * @return {@code true} if the WebView supports multiple windows
      * @see #setSupportMultipleWindows
      */
     public abstract boolean supportMultipleWindows();
 
     /**
-     * Sets the underlying layout algorithm. This will cause a relayout of the
+     * Sets the underlying layout algorithm. This will cause a re-layout of the
      * WebView. The default is {@link LayoutAlgorithm#NARROW_COLUMNS}.
      *
      * @param l the layout algorithm to use, as a {@link LayoutAlgorithm} value
@@ -838,9 +893,9 @@ public abstract class WebSettings {
      * controls loading of all images, including those embedded using the data
      * URI scheme. Use {@link #setBlockNetworkImage} to control loading only
      * of images specified using network URI schemes. Note that if the value of this
-     * setting is changed from false to true, all images resources referenced
+     * setting is changed from {@code false} to {@code true}, all images resources referenced
      * by content currently displayed by the WebView are loaded automatically.
-     * The default is true.
+     * The default is {@code true}.
      *
      * @param flag whether the WebView should load image resources
      */
@@ -850,7 +905,7 @@ public abstract class WebSettings {
      * Gets whether the WebView loads image resources. This includes
      * images embedded using the data URI scheme.
      *
-     * @return true if the WebView loads image resources
+     * @return {@code true} if the WebView loads image resources
      * @see #setLoadsImagesAutomatically
      */
     public abstract boolean getLoadsImagesAutomatically();
@@ -859,12 +914,12 @@ public abstract class WebSettings {
      * Sets whether the WebView should not load image resources from the
      * network (resources accessed via http and https URI schemes).  Note
      * that this method has no effect unless
-     * {@link #getLoadsImagesAutomatically} returns true. Also note that
+     * {@link #getLoadsImagesAutomatically} returns {@code true}. Also note that
      * disabling all network loads using {@link #setBlockNetworkLoads}
      * will also prevent network images from loading, even if this flag is set
-     * to false. When the value of this setting is changed from true to false,
+     * to false. When the value of this setting is changed from {@code true} to {@code false},
      * network images resources referenced by content currently displayed by
-     * the WebView are fetched automatically. The default is false.
+     * the WebView are fetched automatically. The default is {@code false}.
      *
      * @param flag whether the WebView should not load image resources from the
      *             network
@@ -875,7 +930,7 @@ public abstract class WebSettings {
     /**
      * Gets whether the WebView does not load image resources from the network.
      *
-     * @return true if the WebView does not load image resources from the network
+     * @return {@code true} if the WebView does not load image resources from the network
      * @see #setBlockNetworkImage
      */
     public abstract boolean getBlockNetworkImage();
@@ -884,17 +939,17 @@ public abstract class WebSettings {
      * Sets whether the WebView should not load resources from the network.
      * Use {@link #setBlockNetworkImage} to only avoid loading
      * image resources. Note that if the value of this setting is
-     * changed from true to false, network resources referenced by content
+     * changed from {@code true} to {@code false}, network resources referenced by content
      * currently displayed by the WebView are not fetched until
      * {@link android.webkit.WebView#reload} is called.
      * If the application does not have the
      * {@link android.Manifest.permission#INTERNET} permission, attempts to set
-     * a value of false will cause a {@link java.lang.SecurityException}
-     * to be thrown. The default value is false if the application has the
+     * a value of {@code false} will cause a {@link java.lang.SecurityException}
+     * to be thrown. The default value is {@code false} if the application has the
      * {@link android.Manifest.permission#INTERNET} permission, otherwise it is
-     * true.
+     * {@code true}.
      *
-     * @param flag true means block network loads by the WebView
+     * @param flag {@code true} means block network loads by the WebView
      * @see android.webkit.WebView#reload
      */
     public abstract void setBlockNetworkLoads(boolean flag);
@@ -902,16 +957,16 @@ public abstract class WebSettings {
     /**
      * Gets whether the WebView does not load any resources from the network.
      *
-     * @return true if the WebView does not load any resources from the network
+     * @return {@code true} if the WebView does not load any resources from the network
      * @see #setBlockNetworkLoads
      */
     public abstract boolean getBlockNetworkLoads();
 
     /**
      * Tells the WebView to enable JavaScript execution.
-     * <b>The default is false.</b>
+     * <b>The default is {@code false}.</b>
      *
-     * @param flag true if the WebView should execute JavaScript
+     * @param flag {@code true} if the WebView should execute JavaScript
      */
     public abstract void setJavaScriptEnabled(boolean flag);
 
@@ -924,12 +979,12 @@ public abstract class WebSettings {
      * Note that this setting affects only JavaScript access to file scheme
      * resources. Other access to such resources, for example, from image HTML
      * elements, is unaffected. To prevent possible violation of same domain policy
-     * on {@link android.os.Build.VERSION_CODES#ICE_CREAM_SANDWICH} and earlier
-     * devices, you should explicitly set this value to {@code false}.
+     * when targeting {@link android.os.Build.VERSION_CODES#ICE_CREAM_SANDWICH_MR1} and earlier,
+     * you should explicitly set this value to {@code false}.
      * <p>
-     * The default value is true for API level
+     * The default value is {@code true} for apps targeting
      * {@link android.os.Build.VERSION_CODES#ICE_CREAM_SANDWICH_MR1} and below,
-     * and false for API level {@link android.os.Build.VERSION_CODES#JELLY_BEAN}
+     * and {@code false} when targeting {@link android.os.Build.VERSION_CODES#JELLY_BEAN}
      * and above.
      *
      * @param flag whether JavaScript running in the context of a file scheme
@@ -940,18 +995,18 @@ public abstract class WebSettings {
     /**
      * Sets whether JavaScript running in the context of a file scheme URL
      * should be allowed to access content from other file scheme URLs. To
-     * enable the most restrictive, and therefore secure policy, this setting
+     * enable the most restrictive, and therefore secure, policy this setting
      * should be disabled. Note that the value of this setting is ignored if
-     * the value of {@link #getAllowUniversalAccessFromFileURLs} is true.
+     * the value of {@link #getAllowUniversalAccessFromFileURLs} is {@code true}.
      * Note too, that this setting affects only JavaScript access to file scheme
      * resources. Other access to such resources, for example, from image HTML
      * elements, is unaffected. To prevent possible violation of same domain policy
-     * on {@link android.os.Build.VERSION_CODES#ICE_CREAM_SANDWICH} and earlier
-     * devices, you should explicitly set this value to {@code false}.
+     * when targeting {@link android.os.Build.VERSION_CODES#ICE_CREAM_SANDWICH_MR1} and earlier,
+     * you should explicitly set this value to {@code false}.
      * <p>
-     * The default value is true for API level
+     * The default value is {@code true} for apps targeting
      * {@link android.os.Build.VERSION_CODES#ICE_CREAM_SANDWICH_MR1} and below,
-     * and false for API level {@link android.os.Build.VERSION_CODES#JELLY_BEAN}
+     * and {@code false} when targeting {@link android.os.Build.VERSION_CODES#JELLY_BEAN}
      * and above.
      *
      * @param flag whether JavaScript running in the context of a file scheme
@@ -961,9 +1016,9 @@ public abstract class WebSettings {
     public abstract void setAllowFileAccessFromFileURLs(boolean flag);
 
     /**
-     * Sets whether the WebView should enable plugins. The default is false.
+     * Sets whether the WebView should enable plugins. The default is {@code false}.
      *
-     * @param flag true if plugins should be enabled
+     * @param flag {@code true} if plugins should be enabled
      * @deprecated This method has been deprecated in favor of
      *             {@link #setPluginState}
      * @hide Since API level {@link android.os.Build.VERSION_CODES#JELLY_BEAN_MR2}
@@ -980,7 +1035,9 @@ public abstract class WebSettings {
      * {@link PluginState#OFF}.
      *
      * @param state a PluginState value
-     * @deprecated Plugins will not be supported in future, and should not be used.
+     * @deprecated Plugins are not supported in API level
+     *             {@link android.os.Build.VERSION_CODES#KITKAT} or later;
+     *             enabling plugins is a no-op.
      */
     @Deprecated
     public abstract void setPluginState(PluginState state);
@@ -995,6 +1052,7 @@ public abstract class WebSettings {
      * @hide Since API level {@link android.os.Build.VERSION_CODES#JELLY_BEAN_MR2}
      */
     @Deprecated
+    @UnsupportedAppUsage
     public void setPluginsPath(String pluginsPath) {
         // Specified to do nothing, so no need for derived classes to override.
     }
@@ -1028,11 +1086,11 @@ public abstract class WebSettings {
 
     /**
      * Sets whether the Application Caches API should be enabled. The default
-     * is false. Note that in order for the Application Caches API to be
+     * is {@code false}. Note that in order for the Application Caches API to be
      * enabled, a valid database path must also be supplied to
      * {@link #setAppCachePath}.
      *
-     * @param flag true if the WebView should enable Application Caches
+     * @param flag {@code true} if the WebView should enable Application Caches
      */
     public abstract void setAppCacheEnabled(boolean flag);
 
@@ -1072,21 +1130,21 @@ public abstract class WebSettings {
      * page load within a given process, as the WebView implementation may ignore
      * changes to this setting after that point.
      *
-     * @param flag true if the WebView should use the database storage API
+     * @param flag {@code true} if the WebView should use the database storage API
      */
     public abstract void setDatabaseEnabled(boolean flag);
 
     /**
-     * Sets whether the DOM storage API is enabled. The default value is false.
+     * Sets whether the DOM storage API is enabled. The default value is {@code false}.
      *
-     * @param flag true if the WebView should use the DOM storage API
+     * @param flag {@code true} if the WebView should use the DOM storage API
      */
     public abstract void setDomStorageEnabled(boolean flag);
 
     /**
      * Gets whether the DOM Storage APIs are enabled.
      *
-     * @return true if the DOM Storage APIs are enabled
+     * @return {@code true} if the DOM Storage APIs are enabled
      * @see #setDomStorageEnabled
      */
     public abstract boolean getDomStorageEnabled();
@@ -1104,13 +1162,13 @@ public abstract class WebSettings {
     /**
      * Gets whether the database storage API is enabled.
      *
-     * @return true if the database storage API is enabled
+     * @return {@code true} if the database storage API is enabled
      * @see #setDatabaseEnabled
      */
     public abstract boolean getDatabaseEnabled();
 
     /**
-     * Sets whether Geolocation is enabled. The default is true.
+     * Sets whether Geolocation is enabled. The default is {@code true}.
      * <p>
      * Please note that in order for the Geolocation API to be usable
      * by a page in the WebView, the following requirements must be met:
@@ -1132,7 +1190,7 @@ public abstract class WebSettings {
     /**
      * Gets whether JavaScript is enabled.
      *
-     * @return true if JavaScript is enabled
+     * @return {@code true} if JavaScript is enabled
      * @see #setJavaScriptEnabled
      */
     public abstract boolean getJavaScriptEnabled();
@@ -1161,7 +1219,7 @@ public abstract class WebSettings {
     /**
      * Gets whether plugins are enabled.
      *
-     * @return true if plugins are enabled
+     * @return {@code true} if plugins are enabled
      * @see #setPluginsEnabled
      * @deprecated This method has been replaced by {@link #getPluginState}
      * @hide Since API level {@link android.os.Build.VERSION_CODES#JELLY_BEAN_MR2}
@@ -1175,7 +1233,9 @@ public abstract class WebSettings {
      *
      * @return the plugin state as a {@link PluginState} value
      * @see #setPluginState
-     * @deprecated Plugins will not be supported in future, and should not be used.
+     * @deprecated Plugins are not supported in API level
+     *             {@link android.os.Build.VERSION_CODES#KITKAT} or later;
+     *             enabling plugins is a no-op.
      */
     @Deprecated
     public abstract PluginState getPluginState();
@@ -1190,6 +1250,7 @@ public abstract class WebSettings {
      * @hide Since API level {@link android.os.Build.VERSION_CODES#JELLY_BEAN_MR2}
      */
     @Deprecated
+    @UnsupportedAppUsage
     public String getPluginsPath() {
         // Unconditionally returns empty string, so no need for derived classes to override.
         return "";
@@ -1197,17 +1258,17 @@ public abstract class WebSettings {
 
     /**
      * Tells JavaScript to open windows automatically. This applies to the
-     * JavaScript function window.open(). The default is false.
+     * JavaScript function {@code window.open()}. The default is {@code false}.
      *
-     * @param flag true if JavaScript can open windows automatically
+     * @param flag {@code true} if JavaScript can open windows automatically
      */
     public abstract void setJavaScriptCanOpenWindowsAutomatically(boolean flag);
 
     /**
      * Gets whether JavaScript can open windows automatically.
      *
-     * @return true if JavaScript can open windows automatically during
-     *         window.open()
+     * @return {@code true} if JavaScript can open windows automatically during
+     *         {@code window.open()}
      * @see #setJavaScriptCanOpenWindowsAutomatically
      */
     public abstract boolean getJavaScriptCanOpenWindowsAutomatically();
@@ -1229,7 +1290,7 @@ public abstract class WebSettings {
     public abstract String getDefaultTextEncodingName();
 
     /**
-     * Sets the WebView's user-agent string. If the string is null or empty,
+     * Sets the WebView's user-agent string. If the string is {@code null} or empty,
      * the system default value will be used.
      *
      * Note that starting from {@link android.os.Build.VERSION_CODES#KITKAT} Android
@@ -1238,7 +1299,7 @@ public abstract class WebSettings {
      *
      * @param ua new user-agent string
      */
-    public abstract void setUserAgentString(String ua);
+    public abstract void setUserAgentString(@Nullable String ua);
 
     /**
      * Gets the WebView's user-agent string.
@@ -1262,7 +1323,7 @@ public abstract class WebSettings {
     /**
      * Tells the WebView whether it needs to set a node to have focus when
      * {@link WebView#requestFocus(int, android.graphics.Rect)} is called. The
-     * default value is true.
+     * default value is {@code true}.
      *
      * @param flag whether the WebView needs to set a node
      */
@@ -1355,7 +1416,7 @@ public abstract class WebSettings {
     /**
      * Gets whether a video overlay will be used for embedded encrypted video.
      *
-     * @return true if WebView uses a video overlay for embedded encrypted video.
+     * @return {@code true} if WebView uses a video overlay for embedded encrypted video.
      * @see #setVideoOverlayForEmbeddedEncryptedVideoEnabled
      * @hide
      */
@@ -1380,49 +1441,68 @@ public abstract class WebSettings {
     /**
      * Gets whether this WebView should raster tiles when it is
      * offscreen but attached to a window.
-     * @return true if this WebView will raster tiles when it is
+     * @return {@code true} if this WebView will raster tiles when it is
      * offscreen but attached to a window.
      */
     public abstract boolean getOffscreenPreRaster();
 
 
     /**
-     * Sets whether Safe Browsing is enabled. Safe browsing allows WebView to
+     * Sets whether Safe Browsing is enabled. Safe Browsing allows WebView to
      * protect against malware and phishing attacks by verifying the links.
      *
      * <p>
-     * Safe browsing is disabled by default. The recommended way to enable Safe browsing is using a
-     * manifest tag to change the default value to enabled for all WebViews (read <a
-     * href="{@docRoot}reference/android/webkit/WebView.html">general Safe Browsing info</a>).
-     * </p>
+     * Safe Browsing can be disabled for all WebViews using a manifest tag (read <a
+     * href="{@docRoot}reference/android/webkit/WebView.html">general Safe Browsing info</a>). The
+     * manifest tag has a lower precedence than this API.
      *
      * <p>
-     * This API overrides the manifest tag value for this WebView.
-     * </p>
+     * Safe Browsing is enabled by default for devices which support it.
      *
-     * @param enabled Whether Safe browsing is enabled.
+     * @param enabled Whether Safe Browsing is enabled.
      */
     public abstract void setSafeBrowsingEnabled(boolean enabled);
 
     /**
-     * Gets whether Safe browsing is enabled.
+     * Gets whether Safe Browsing is enabled.
      * See {@link #setSafeBrowsingEnabled}.
      *
-     * @return true if Safe browsing is enabled and false otherwise.
+     * @return {@code true} if Safe Browsing is enabled and {@code false} otherwise.
      */
     public abstract boolean getSafeBrowsingEnabled();
 
 
+    /**
+     * Set the force dark mode for this WebView.
+     *
+     * @param forceDark the force dark mode to set.
+     * @see #getForceDark
+     */
+    public void setForceDark(@ForceDark int forceDark) {
+        // Stub implementation to satisfy Roboelectrc shadows that don't override this yet.
+    }
+
+    /**
+     * Get the force dark mode for this WebView.
+     * The default force dark mode is {@link #FORCE_DARK_AUTO}.
+     *
+     * @return the currently set force dark mode.
+     * @see #setForceDark
+     */
+    public @ForceDark int getForceDark() {
+        // Stub implementation to satisfy Roboelectrc shadows that don't override this yet.
+        return FORCE_DARK_AUTO;
+    }
+
     /**
      * @hide
      */
-    @IntDef(flag = true,
-            value = {
-                    MENU_ITEM_NONE,
-                    MENU_ITEM_SHARE,
-                    MENU_ITEM_WEB_SEARCH,
-                    MENU_ITEM_PROCESS_TEXT
-            })
+    @IntDef(flag = true, prefix = { "MENU_ITEM_" }, value = {
+            MENU_ITEM_NONE,
+            MENU_ITEM_SHARE,
+            MENU_ITEM_WEB_SEARCH,
+            MENU_ITEM_PROCESS_TEXT
+    })
     @Retention(RetentionPolicy.SOURCE)
     @Target({ElementType.PARAMETER, ElementType.METHOD})
     private @interface MenuItemFlags {}
@@ -1442,34 +1522,34 @@ public abstract class WebSettings {
     public abstract @MenuItemFlags int getDisabledActionModeMenuItems();
 
     /**
-     * Used with {@link #setDisabledActionModeMenuItems}.
-     *
      * No menu items should be disabled.
+     *
+     * @see #setDisabledActionModeMenuItems
      */
     public static final int MENU_ITEM_NONE = 0;
 
     /**
-     * Used with {@link #setDisabledActionModeMenuItems}.
-     *
      * Disable menu item "Share".
+     *
+     * @see #setDisabledActionModeMenuItems
      */
     public static final int MENU_ITEM_SHARE = 1 << 0;
 
     /**
-     * Used with {@link #setDisabledActionModeMenuItems}.
-     *
      * Disable menu item "Web Search".
+     *
+     * @see #setDisabledActionModeMenuItems
      */
     public static final int MENU_ITEM_WEB_SEARCH = 1 << 1;
 
     /**
-     * Used with {@link #setDisabledActionModeMenuItems}.
-     *
      * Disable all the action mode menu items for text processing.
      * By default WebView searches for activities that are able to handle
      * {@link android.content.Intent#ACTION_PROCESS_TEXT} and show them in the
      * action mode menu. If this flag is set via {@link
      * #setDisabledActionModeMenuItems}, these menu items will be disabled.
+     *
+     * @see #setDisabledActionModeMenuItems
      */
     public static final int MENU_ITEM_PROCESS_TEXT = 1 << 2;
 }
