@@ -105,6 +105,15 @@ public class EditText extends TextView {
 
     @Override
     public Editable getText() {
+        CharSequence text = super.getText();
+        // This can only happen during construction.
+        if (text == null) {
+            return null;
+        }
+        if (text instanceof Editable) {
+            return (Editable) super.getText();
+        }
+        super.setText(text, BufferType.EDITABLE);
         return (Editable) super.getText();
     }
 
