@@ -24,10 +24,12 @@ import android.annotation.Nullable;
 import android.annotation.StringRes;
 import android.annotation.StyleRes;
 import android.annotation.TestApi;
+import android.annotation.UnsupportedAppUsage;
 import android.app.ActionBar;
 import android.content.Context;
 import android.content.res.TypedArray;
 import android.graphics.drawable.Drawable;
+import android.os.Build;
 import android.os.Parcel;
 import android.os.Parcelable;
 import android.text.Layout;
@@ -43,6 +45,7 @@ import android.view.MotionEvent;
 import android.view.View;
 import android.view.ViewGroup;
 import android.view.ViewParent;
+import android.view.inspector.InspectableProperty;
 
 import com.android.internal.R;
 import com.android.internal.view.menu.MenuBuilder;
@@ -133,8 +136,10 @@ public class Toolbar extends ViewGroup {
     private static final String TAG = "Toolbar";
 
     private ActionMenuView mMenuView;
+    @UnsupportedAppUsage
     private TextView mTitleTextView;
     private TextView mSubtitleTextView;
+    @UnsupportedAppUsage(maxTargetSdk = Build.VERSION_CODES.P, trackingBug = 115609023)
     private ImageButton mNavButtonView;
     private ImageView mLogoView;
 
@@ -157,9 +162,13 @@ public class Toolbar extends ViewGroup {
 
     private int mMaxButtonHeight;
 
+    @UnsupportedAppUsage
     private int mTitleMarginStart;
+    @UnsupportedAppUsage
     private int mTitleMarginEnd;
+    @UnsupportedAppUsage
     private int mTitleMarginTop;
+    @UnsupportedAppUsage
     private int mTitleMarginBottom;
 
     private RtlSpacingHelper mContentInsets;
@@ -228,6 +237,8 @@ public class Toolbar extends ViewGroup {
 
         final TypedArray a = context.obtainStyledAttributes(attrs, R.styleable.Toolbar,
                 defStyleAttr, defStyleRes);
+        saveAttributeDataForStyleable(context, R.styleable.Toolbar,
+                attrs, a, defStyleAttr, defStyleRes);
 
         mTitleTextAppearance = a.getResourceId(R.styleable.Toolbar_titleTextAppearance, 0);
         mSubtitleTextAppearance = a.getResourceId(R.styleable.Toolbar_subtitleTextAppearance, 0);
@@ -375,6 +386,7 @@ public class Toolbar extends ViewGroup {
      *         0 if menus are inflated against the toolbar theme
      * @see #setPopupTheme(int)
      */
+    @InspectableProperty
     public int getPopupTheme() {
         return mPopupTheme;
     }
@@ -406,6 +418,7 @@ public class Toolbar extends ViewGroup {
      * @see #setTitleMarginStart(int)
      * @attr ref android.R.styleable#Toolbar_titleMarginStart
      */
+    @InspectableProperty
     public int getTitleMarginStart() {
         return mTitleMarginStart;
     }
@@ -428,6 +441,7 @@ public class Toolbar extends ViewGroup {
      * @see #setTitleMarginTop(int)
      * @attr ref android.R.styleable#Toolbar_titleMarginTop
      */
+    @InspectableProperty
     public int getTitleMarginTop() {
         return mTitleMarginTop;
     }
@@ -450,6 +464,7 @@ public class Toolbar extends ViewGroup {
      * @see #setTitleMarginEnd(int)
      * @attr ref android.R.styleable#Toolbar_titleMarginEnd
      */
+    @InspectableProperty
     public int getTitleMarginEnd() {
         return mTitleMarginEnd;
     }
@@ -472,6 +487,7 @@ public class Toolbar extends ViewGroup {
      * @see #setTitleMarginBottom(int)
      * @attr ref android.R.styleable#Toolbar_titleMarginBottom
      */
+    @InspectableProperty
     public int getTitleMarginBottom() {
         return mTitleMarginBottom;
     }
@@ -642,6 +658,7 @@ public class Toolbar extends ViewGroup {
      * @see #setLogo(int)
      * @see #setLogo(android.graphics.drawable.Drawable)
      */
+    @InspectableProperty
     public Drawable getLogo() {
         return mLogoView != null ? mLogoView.getDrawable() : null;
     }
@@ -680,6 +697,7 @@ public class Toolbar extends ViewGroup {
      *
      * @return A description of the logo
      */
+    @InspectableProperty
     public CharSequence getLogoDescription() {
         return mLogoView != null ? mLogoView.getContentDescription() : null;
     }
@@ -727,6 +745,7 @@ public class Toolbar extends ViewGroup {
      *
      * @return The current title.
      */
+    @InspectableProperty
     public CharSequence getTitle() {
         return mTitleText;
     }
@@ -783,6 +802,7 @@ public class Toolbar extends ViewGroup {
      *
      * @return The current subtitle
      */
+    @InspectableProperty
     public CharSequence getSubtitle() {
         return mSubtitleText;
     }
@@ -887,6 +907,7 @@ public class Toolbar extends ViewGroup {
      *
      * @attr ref android.R.styleable#Toolbar_navigationContentDescription
      */
+    @InspectableProperty
     @Nullable
     public CharSequence getNavigationContentDescription() {
         return mNavButtonView != null ? mNavButtonView.getContentDescription() : null;
@@ -979,6 +1000,7 @@ public class Toolbar extends ViewGroup {
      *
      * @attr ref android.R.styleable#Toolbar_navigationIcon
      */
+    @InspectableProperty
     @Nullable
     public Drawable getNavigationIcon() {
         return mNavButtonView != null ? mNavButtonView.getDrawable() : null;
@@ -1007,6 +1029,100 @@ public class Toolbar extends ViewGroup {
         return mNavButtonView;
     }
 
+    /**
+     * Retrieve the currently configured content description for the collapse button view.
+     * This will be used to describe the collapse action to users through mechanisms such
+     * as screen readers or tooltips.
+     *
+     * @return The collapse button's content description
+     *
+     * @attr ref android.R.styleable#Toolbar_collapseContentDescription
+     */
+    @InspectableProperty
+    @Nullable
+    public CharSequence getCollapseContentDescription() {
+        return mCollapseButtonView != null ? mCollapseButtonView.getContentDescription() : null;
+    }
+
+    /**
+     * Set a content description for the collapse button if one is present. The content description
+     * will be read via screen readers or other accessibility systems to explain the action of the
+     * collapse button.
+     *
+     * @param resId Resource ID of a content description string to set, or 0 to
+     *              clear the description
+     *
+     * @attr ref android.R.styleable#Toolbar_collapseContentDescription
+     */
+    public void setCollapseContentDescription(@StringRes int resId) {
+        setCollapseContentDescription(resId != 0 ? getContext().getText(resId) : null);
+    }
+
+    /**
+     * Set a content description for the collapse button if one is present. The content description
+     * will be read via screen readers or other accessibility systems to explain the action of the
+     * navigation button.
+     *
+     * @param description Content description to set, or <code>null</code> to
+     *                    clear the content description
+     *
+     * @attr ref android.R.styleable#Toolbar_collapseContentDescription
+     */
+    public void setCollapseContentDescription(@Nullable CharSequence description) {
+        if (!TextUtils.isEmpty(description)) {
+            ensureCollapseButtonView();
+        }
+        if (mCollapseButtonView != null) {
+            mCollapseButtonView.setContentDescription(description);
+        }
+    }
+
+    /**
+     * Return the current drawable used as the collapse icon.
+     *
+     * @return The collapse icon drawable
+     *
+     * @attr ref android.R.styleable#Toolbar_collapseIcon
+     */
+    @InspectableProperty
+    @Nullable
+    public Drawable getCollapseIcon() {
+        return mCollapseButtonView != null ? mCollapseButtonView.getDrawable() : null;
+    }
+
+    /**
+     * Set the icon to use for the toolbar's collapse button.
+     *
+     * <p>The collapse button appears at the start of the toolbar when an action view is present
+     * .</p>
+     *
+     * @param resId Resource ID of a drawable to set
+     *
+     * @attr ref android.R.styleable#Toolbar_collapseIcon
+     */
+    public void setCollapseIcon(@DrawableRes int resId) {
+        setCollapseIcon(getContext().getDrawable(resId));
+    }
+
+    /**
+     * Set the icon to use for the toolbar's collapse button.
+     *
+     * <p>The collapse button appears at the start of the toolbar when an action view is present
+     * .</p>
+     *
+     * @param icon Drawable to set, may be null to use the default icon
+     *
+     * @attr ref android.R.styleable#Toolbar_collapseIcon
+     */
+    public void setCollapseIcon(@Nullable Drawable icon) {
+        if (icon != null) {
+            ensureCollapseButtonView();
+            mCollapseButtonView.setImageDrawable(icon);
+        } else if (mCollapseButtonView != null) {
+            mCollapseButtonView.setImageDrawable(mCollapseIcon);
+        }
+    }
+
     /**
      * Return the Menu shown in the toolbar.
      *
@@ -1134,6 +1250,7 @@ public class Toolbar extends ViewGroup {
      * @see #getContentInsetRight()
      * @attr ref android.R.styleable#Toolbar_contentInsetStart
      */
+    @InspectableProperty
     public int getContentInsetStart() {
         return mContentInsets != null ? mContentInsets.getStart() : 0;
     }
@@ -1154,6 +1271,7 @@ public class Toolbar extends ViewGroup {
      * @see #getContentInsetRight()
      * @attr ref android.R.styleable#Toolbar_contentInsetEnd
      */
+    @InspectableProperty
     public int getContentInsetEnd() {
         return mContentInsets != null ? mContentInsets.getEnd() : 0;
     }
@@ -1197,6 +1315,7 @@ public class Toolbar extends ViewGroup {
      * @see #getContentInsetRight()
      * @attr ref android.R.styleable#Toolbar_contentInsetLeft
      */
+    @InspectableProperty
     public int getContentInsetLeft() {
         return mContentInsets != null ? mContentInsets.getLeft() : 0;
     }
@@ -1217,6 +1336,7 @@ public class Toolbar extends ViewGroup {
      * @see #getContentInsetLeft()
      * @attr ref android.R.styleable#Toolbar_contentInsetRight
      */
+    @InspectableProperty
     public int getContentInsetRight() {
         return mContentInsets != null ? mContentInsets.getRight() : 0;
     }
@@ -1233,6 +1353,7 @@ public class Toolbar extends ViewGroup {
      * @see #setContentInsetStartWithNavigation(int)
      * @attr ref android.R.styleable#Toolbar_contentInsetStartWithNavigation
      */
+    @InspectableProperty
     public int getContentInsetStartWithNavigation() {
         return mContentInsetStartWithNavigation != RtlSpacingHelper.UNDEFINED
                 ? mContentInsetStartWithNavigation
@@ -1276,6 +1397,7 @@ public class Toolbar extends ViewGroup {
      * @see #setContentInsetEndWithActions(int)
      * @attr ref android.R.styleable#Toolbar_contentInsetEndWithActions
      */
+    @InspectableProperty
     public int getContentInsetEndWithActions() {
         return mContentInsetEndWithActions != RtlSpacingHelper.UNDEFINED
                 ? mContentInsetEndWithActions
@@ -2262,7 +2384,7 @@ public class Toolbar extends ViewGroup {
             out.writeInt(isOverflowOpen ? 1 : 0);
         }
 
-        public static final Creator<SavedState> CREATOR = new Creator<SavedState>() {
+        public static final @android.annotation.NonNull Creator<SavedState> CREATOR = new Creator<SavedState>() {
 
             @Override
             public SavedState createFromParcel(Parcel source) {
