@@ -16,11 +16,18 @@
 
 package com.android.internal.os;
 
+import android.annotation.UnsupportedAppUsage;
 import android.content.Context;
 import android.os.Handler;
 import android.os.Looper;
 import android.os.Message;
 
+/**
+ * @deprecated Use {@link com.android.internal.util.function.pooled.PooledLambda#obtainMessage}
+ *      to achieve the same effect of storing multiple values in a message with the added typesafety
+ *      and code continuity benefits.
+ */
+@Deprecated
 public class HandlerCaller {
     final Looper mMainLooper;
     final Handler mH;
@@ -82,6 +89,7 @@ public class HandlerCaller {
         mH.removeMessages(what, obj);
     }
     
+    @UnsupportedAppUsage
     public void sendMessage(Message msg) {
         mH.sendMessage(msg);
     }
@@ -106,6 +114,7 @@ public class HandlerCaller {
         return args;
     }
 
+    @UnsupportedAppUsage
     public Message obtainMessage(int what) {
         return mH.obtainMessage(what);
     }
@@ -121,6 +130,7 @@ public class HandlerCaller {
         return mH.obtainMessage(what, arg1 ? 1 : 0, 0, args);
     }
     
+    @UnsupportedAppUsage
     public Message obtainMessageO(int what, Object arg1) {
         return mH.obtainMessage(what, 0, 0, arg1);
     }
@@ -133,6 +143,7 @@ public class HandlerCaller {
         return mH.obtainMessage(what, arg1, arg2);
     }
     
+    @UnsupportedAppUsage
     public Message obtainMessageIO(int what, int arg1, Object arg2) {
         return mH.obtainMessage(what, arg1, 0, arg2);
     }
@@ -149,6 +160,7 @@ public class HandlerCaller {
         return mH.obtainMessage(what, arg1, arg2, args);
     }
     
+    @UnsupportedAppUsage
     public Message obtainMessageIOO(int what, int arg1, Object arg2, Object arg3) {
         SomeArgs args = SomeArgs.obtain();
         args.arg1 = arg2;
@@ -183,6 +195,7 @@ public class HandlerCaller {
         return mH.obtainMessage(what, arg1, arg2, args);
     }
 
+    @UnsupportedAppUsage
     public Message obtainMessageOO(int what, Object arg1, Object arg2) {
         SomeArgs args = SomeArgs.obtain();
         args.arg1 = arg1;
@@ -190,6 +203,7 @@ public class HandlerCaller {
         return mH.obtainMessage(what, 0, 0, args);
     }
     
+    @UnsupportedAppUsage
     public Message obtainMessageOOO(int what, Object arg1, Object arg2, Object arg3) {
         SomeArgs args = SomeArgs.obtain();
         args.arg1 = arg1;
