@@ -16,10 +16,12 @@
 
 package com.android.internal.util;
 
+import android.annotation.UnsupportedAppUsage;
 import android.content.ComponentName;
 import android.content.Context;
 import android.content.Intent;
 import android.content.ServiceConnection;
+import android.os.Build;
 import android.os.Handler;
 import android.os.HandlerThread;
 import android.os.IBinder;
@@ -105,6 +107,7 @@ public class AsyncChannel {
      * msg.obj  == the AsyncChannel
      * msg.replyTo == dstMessenger if successful
      */
+    @UnsupportedAppUsage
     public static final int CMD_CHANNEL_HALF_CONNECTED = BASE + 0;
 
     /**
@@ -114,6 +117,7 @@ public class AsyncChannel {
      *
      * msg.replyTo = srcMessenger.
      */
+    @UnsupportedAppUsage
     public static final int CMD_CHANNEL_FULL_CONNECTION = BASE + 1;
 
     /**
@@ -159,6 +163,7 @@ public class AsyncChannel {
         sCmdToString[CMD_CHANNEL_DISCONNECT - BASE] = "CMD_CHANNEL_DISCONNECT";
         sCmdToString[CMD_CHANNEL_DISCONNECTED - BASE] = "CMD_CHANNEL_DISCONNECTED";
     }
+    @UnsupportedAppUsage(maxTargetSdk = Build.VERSION_CODES.P, trackingBug = 115609023)
     protected static String cmdToString(int cmd) {
         cmd -= BASE;
         if ((cmd >= 0) && (cmd < sCmdToString.length)) {
@@ -169,6 +174,7 @@ public class AsyncChannel {
     }
 
     /** Successful status always 0, !0 is an unsuccessful status */
+    @UnsupportedAppUsage
     public static final int STATUS_SUCCESSFUL = 0;
 
     /** Error attempting to bind on a connect */
@@ -204,6 +210,7 @@ public class AsyncChannel {
     /**
      * AsyncChannel constructor
      */
+    @UnsupportedAppUsage
     public AsyncChannel() {
     }
 
@@ -255,6 +262,7 @@ public class AsyncChannel {
      *
      * @return STATUS_SUCCESSFUL on success any other value is an error.
      */
+    @UnsupportedAppUsage
     public int connectSync(Context srcContext, Handler srcHandler, Messenger dstMessenger) {
         if (DBG) log("halfConnectSync srcHandler to the dstMessenger  E");
 
@@ -370,6 +378,7 @@ public class AsyncChannel {
      * @param srcHandler
      * @param dstMessenger
      */
+    @UnsupportedAppUsage
     public void connect(Context srcContext, Handler srcHandler, Messenger dstMessenger) {
         if (DBG) log("connect srcHandler to the dstMessenger  E");
 
@@ -392,6 +401,7 @@ public class AsyncChannel {
      * @param srcHandler
      * @param dstMessenger
      */
+    @UnsupportedAppUsage
     public void connected(Context srcContext, Handler srcHandler, Messenger dstMessenger) {
         if (DBG) log("connected srcHandler to the dstMessenger  E");
 
@@ -446,6 +456,7 @@ public class AsyncChannel {
     /**
      * Disconnect
      */
+    @UnsupportedAppUsage
     public void disconnect() {
         if ((mConnection != null) && (mSrcContext != null)) {
             mSrcContext.unbindService(mConnection);
@@ -475,6 +486,7 @@ public class AsyncChannel {
      *
      * @param msg
      */
+    @UnsupportedAppUsage
     public void sendMessage(Message msg) {
         msg.replyTo = mSrcMessenger;
         try {
@@ -489,6 +501,7 @@ public class AsyncChannel {
      *
      * @param what
      */
+    @UnsupportedAppUsage
     public void sendMessage(int what) {
         Message msg = Message.obtain();
         msg.what = what;
@@ -501,6 +514,7 @@ public class AsyncChannel {
      * @param what
      * @param arg1
      */
+    @UnsupportedAppUsage
     public void sendMessage(int what, int arg1) {
         Message msg = Message.obtain();
         msg.what = what;
@@ -515,6 +529,7 @@ public class AsyncChannel {
      * @param arg1
      * @param arg2
      */
+    @UnsupportedAppUsage
     public void sendMessage(int what, int arg1, int arg2) {
         Message msg = Message.obtain();
         msg.what = what;
@@ -531,6 +546,7 @@ public class AsyncChannel {
      * @param arg2
      * @param obj
      */
+    @UnsupportedAppUsage
     public void sendMessage(int what, int arg1, int arg2, Object obj) {
         Message msg = Message.obtain();
         msg.what = what;
@@ -559,6 +575,7 @@ public class AsyncChannel {
      * @param srcMsg
      * @param dstMsg
      */
+    @UnsupportedAppUsage
     public void replyToMessage(Message srcMsg, Message dstMsg) {
         try {
             dstMsg.replyTo = mSrcMessenger;
@@ -575,6 +592,7 @@ public class AsyncChannel {
      * @param srcMsg
      * @param what
      */
+    @UnsupportedAppUsage(maxTargetSdk = Build.VERSION_CODES.P, trackingBug = 115609023)
     public void replyToMessage(Message srcMsg, int what) {
         Message msg = Message.obtain();
         msg.what = what;
@@ -588,6 +606,7 @@ public class AsyncChannel {
      * @param what
      * @param arg1
      */
+    @UnsupportedAppUsage
     public void replyToMessage(Message srcMsg, int what, int arg1) {
         Message msg = Message.obtain();
         msg.what = what;
@@ -620,6 +639,7 @@ public class AsyncChannel {
      * @param arg2
      * @param obj
      */
+    @UnsupportedAppUsage
     public void replyToMessage(Message srcMsg, int what, int arg1, int arg2, Object obj) {
         Message msg = Message.obtain();
         msg.what = what;
@@ -636,6 +656,7 @@ public class AsyncChannel {
      * @param what
      * @param obj
      */
+    @UnsupportedAppUsage
     public void replyToMessage(Message srcMsg, int what, Object obj) {
         Message msg = Message.obtain();
         msg.what = what;
@@ -649,6 +670,7 @@ public class AsyncChannel {
      * @param msg to send
      * @return reply message or null if an error.
      */
+    @UnsupportedAppUsage
     public Message sendMessageSynchronously(Message msg) {
         Message resultMsg = SyncMessenger.sendMessageSynchronously(mDstMessenger, msg);
         return resultMsg;
@@ -690,6 +712,7 @@ public class AsyncChannel {
      * @param arg2
      * @return reply message or null if an error.
      */
+    @UnsupportedAppUsage
     public Message sendMessageSynchronously(int what, int arg1, int arg2) {
         Message msg = Message.obtain();
         msg.what = what;
