@@ -16,15 +16,19 @@
 
 package com.android.internal.view;
 
+import android.graphics.Point;
 import android.graphics.Rect;
 import android.hardware.input.InputManager;
 import android.os.Bundle;
 import android.os.ParcelFileDescriptor;
 import android.os.RemoteException;
 import android.util.MergedConfiguration;
+import android.view.DisplayCutout;
 import android.view.DragEvent;
 import android.view.IWindow;
 import android.view.IWindowSession;
+import android.view.InsetsSourceControl;
+import android.view.InsetsState;
 import android.view.PointerIcon;
 
 import com.android.internal.os.IResultReceiver;
@@ -41,7 +45,8 @@ public class BaseIWindow extends IWindow.Stub {
     public void resized(Rect frame, Rect overscanInsets, Rect contentInsets, Rect visibleInsets,
             Rect stableInsets, Rect outsets, boolean reportDraw,
             MergedConfiguration mergedConfiguration, Rect backDropFrame, boolean forceLayout,
-            boolean alwaysConsumeNavBar, int displayId) {
+            boolean alwaysConsumeSystemBars, int displayId,
+            DisplayCutout.ParcelableWrapper displayCutout) {
         if (reportDraw) {
             try {
                 mSession.finishDrawing(this);
@@ -50,6 +55,19 @@ public class BaseIWindow extends IWindow.Stub {
         }
     }
 
+    @Override
+    public void locationInParentDisplayChanged(Point offset) {
+    }
+
+    @Override
+    public void insetsChanged(InsetsState insetsState) {
+    }
+
+    @Override
+    public void insetsControlChanged(InsetsState insetsState,
+            InsetsSourceControl[] activeControls) throws RemoteException {
+    }
+
     @Override
     public void moved(int newX, int newY) {
     }
